from _typeshed import Incomplete
from rdkit import Chem as Chem, Geometry as Geometry
from rdkit.Chem import AllChem as AllChem
from rdkit.Chem.Subshape import (
    BuilderUtils as BuilderUtils,
    SubshapeObjects as SubshapeObjects,
)

class SubshapeCombineOperations:
    UNION: int
    SUM: int
    INTERSECT: int

class SubshapeBuilder:
    gridDims: Incomplete
    gridSpacing: float
    winRad: float
    nbrCount: int
    terminalPtRadScale: float
    fraction: float
    stepSize: float
    featFactory: Incomplete
    def SampleSubshape(self, subshape1, newSpacing): ...
    def GenerateSubshapeShape(
        self, cmpd, confId: int = ..., addSkeleton: bool = ..., **kwargs
    ): ...
    def __call__(self, cmpd, **kwargs): ...
    def GenerateSubshapeSkeleton(
        self,
        shape,
        conf: Incomplete | None = ...,
        terminalPtsOnly: bool = ...,
        skelFromConf: bool = ...,
    ) -> None: ...
    def CombineSubshapes(self, subshape1, subshape2, operation=...): ...



from _typeshed import Incomplete

class DescriptorCalculationError(Exception): ...
class ClassificationError(Exception): ...

class ModelPackage:
    _descCalc: Incomplete
    _model: Incomplete
    _notes: Incomplete
    _dataSet: Incomplete
    _initialized: int
    _supplementalData: Incomplete
    def __init__(
        self,
        descCalc: Incomplete | None = ...,
        model: Incomplete | None = ...,
        dataSet: Incomplete | None = ...,
        notes: str = ...,
    ) -> None: ...
    def SetCalculator(self, calc) -> None: ...
    def GetCalculator(self): ...
    def SetModel(self, model) -> None: ...
    def GetModel(self): ...
    def SetDataset(self, data) -> None: ...
    def GetDataset(self): ...
    def SetNotes(self, notes) -> None: ...
    def GetNotes(self): ...
    def SetSupplementalData(self, suppD) -> None: ...
    def GetSupplementalData(self): ...
    def AddSupplementalData(self, data) -> None: ...
    def Classify(self, obj, label: str = ..., threshold: int = ...): ...
    def Init(self) -> None: ...



from _typeshed import Incomplete
from rdkit.ML.DecTree import DecTree as DecTree
from rdkit.ML.InfoTheory import entropy as entropy

def CalcTotalEntropy(examples, nPossibleVals): ...
def GenVarTable(examples, nPossibleVals, vars): ...
def ID3(
    examples,
    target,
    attrs,
    nPossibleVals,
    depth: int = ...,
    maxDepth: int = ...,
    **kwargs
): ...
def ID3Boot(
    examples,
    attrs,
    nPossibleVals,
    initialVar: Incomplete | None = ...,
    depth: int = ...,
    maxDepth: int = ...,
    **kwargs
): ...



from _typeshed import Incomplete

def StandardizeMatrix(mat): ...
def FormCovarianceMatrix(mat): ...
def FormCorrelationMatrix(mat): ...
def PrincipalComponents(mat, reverseOrder: int = ...): ...
def TransformPoints(tFormMat, pts): ...
def MeanAndDev(vect, sampleSD: int = ...): ...
def R2(orig, residSum): ...

tConfs: Incomplete
tTable: Incomplete

def GetConfidenceInterval(sd, n, level: int = ...): ...



from _typeshed import Incomplete
from rdkit.ML.DecTree import CrossValidate as CrossValidate, PruneTree as PruneTree

class Forest:
    def MakeHistogram(self): ...
    def CollectVotes(self, example): ...
    treeVotes: Incomplete
    def ClassifyExample(self, example): ...
    def GetVoteDetails(self): ...
    _nPossible: Incomplete
    def Grow(
        self,
        examples,
        attrs,
        nPossibleVals,
        nTries: int = ...,
        pruneIt: int = ...,
        lessGreedy: int = ...,
    ) -> None: ...
    def Pickle(self, fileName: str = ...) -> None: ...
    def AddTree(self, tree, error) -> None: ...
    errList: Incomplete
    def AverageErrors(self) -> None: ...
    treeList: Incomplete
    countList: Incomplete
    def SortTrees(self, sortOnError: int = ...) -> None: ...
    def GetTree(self, i): ...
    def SetTree(self, i, val) -> None: ...
    def GetCount(self, i): ...
    def SetCount(self, i, val) -> None: ...
    def GetError(self, i): ...
    def SetError(self, i, val) -> None: ...
    def GetDataTuple(self, i): ...
    def SetDataTuple(self, i, tup) -> None: ...
    def GetAllData(self): ...
    def __len__(self) -> int: ...
    def __getitem__(self, which): ...
    def __str__(self): ...
    def __init__(self) -> None: ...



from _typeshed import Incomplete
from rdkit.VLib.Node import VLibNode as VLibNode

class FilterNode(VLibNode):
    _func: Incomplete
    _negate: Incomplete
    def __init__(
        self, func: Incomplete | None = ..., negate: int = ..., **kwargs
    ) -> None: ...
    def SetNegate(self, state) -> None: ...
    def Negate(self): ...
    def next(self): ...

def _runDoctests(verbose: Incomplete | None = ...) -> None: ...



from _typeshed import Incomplete

class LazySig:
    computeFunc: Incomplete
    size: Incomplete
    _cache: Incomplete
    def __init__(self, computeFunc, sigSize) -> None: ...
    def __len__(self) -> int: ...
    def __getitem__(self, which): ...

def _runDoctests(verbose: Incomplete | None = ...) -> None: ...



from _typeshed import Incomplete
from rdkit.Dbase import DbInfo as DbInfo, DbModule as DbModule, DbUtils as DbUtils

class DbError(RuntimeError): ...

class DbConnect:
    dbName: Incomplete
    tableName: Incomplete
    user: Incomplete
    password: Incomplete
    cn: Incomplete
    cursor: Incomplete
    def __init__(
        self,
        dbName: str = ...,
        tableName: str = ...,
        user: str = ...,
        password: str = ...,
    ) -> None: ...
    def GetTableNames(self, includeViews: int = ...): ...
    def GetColumnNames(
        self,
        table: str = ...,
        join: str = ...,
        what: str = ...,
        where: str = ...,
        **kwargs
    ): ...
    def GetColumnNamesAndTypes(
        self,
        table: str = ...,
        join: str = ...,
        what: str = ...,
        where: str = ...,
        **kwargs
    ): ...
    def GetColumns(self, fields, table: str = ..., join: str = ..., **kwargs): ...
    def GetData(
        self,
        table: Incomplete | None = ...,
        fields: str = ...,
        where: str = ...,
        removeDups: int = ...,
        join: str = ...,
        transform: Incomplete | None = ...,
        randomAccess: int = ...,
        **kwargs
    ): ...
    def GetDataCount(
        self,
        table: Incomplete | None = ...,
        where: str = ...,
        join: str = ...,
        **kwargs
    ): ...
    def GetCursor(self): ...
    def KillCursor(self) -> None: ...
    def AddTable(self, tableName, colString) -> None: ...
    def InsertData(self, tableName, vals) -> None: ...
    def InsertColumnData(self, tableName, columnName, value, where) -> None: ...
    def AddColumn(self, tableName, colName, colType) -> None: ...
    def Commit(self) -> None: ...



from typing import Any, ClassVar

boostVersion: str
rdkitBuild: str
rdkitVersion: str

class BlockLogs:
    __instance_size__: ClassVar[int] = ...
    @classmethod
    def __init__(cls, *args, **kwargs) -> None: ...
    @classmethod
    def __enter__(cls) -> Any: ...
    @classmethod
    def __exit__(cls, type, value, traceback) -> Any: ...
    @classmethod
    def __reduce__(cls) -> Any: ...

class _listSt6vectorIiSaIiEE:
    __instance_size__: ClassVar[int] = ...
    @classmethod
    def __init__(cls, *args, **kwargs) -> None: ...
    @classmethod
    def __contains__(cls, other) -> Any: ...
    @classmethod
    def __delitem__(cls, other) -> Any: ...
    @classmethod
    def __getitem__(cls, index) -> Any: ...
    @classmethod
    def __iter__(cls) -> Any: ...
    @classmethod
    def __len__(cls) -> Any: ...
    @classmethod
    def __reduce__(cls) -> Any: ...
    @classmethod
    def __setitem__(cls, index, object) -> Any: ...

class _listSt6vectorIjSaIjEE:
    __instance_size__: ClassVar[int] = ...
    @classmethod
    def __init__(cls, *args, **kwargs) -> None: ...
    @classmethod
    def __contains__(cls, other) -> Any: ...
    @classmethod
    def __delitem__(cls, other) -> Any: ...
    @classmethod
    def __getitem__(cls, index) -> Any: ...
    @classmethod
    def __iter__(cls) -> Any: ...
    @classmethod
    def __len__(cls) -> Any: ...
    @classmethod
    def __reduce__(cls) -> Any: ...
    @classmethod
    def __setitem__(cls, index, object) -> Any: ...

class _listi:
    __instance_size__: ClassVar[int] = ...
    @classmethod
    def __init__(cls, *args, **kwargs) -> None: ...
    @classmethod
    def __contains__(cls, other) -> Any: ...
    @classmethod
    def __delitem__(cls, other) -> Any: ...
    @classmethod
    def __getitem__(cls, index) -> Any: ...
    @classmethod
    def __iter__(cls, boost, std) -> Any: ...
    @classmethod
    def __len__(cls) -> Any: ...
    @classmethod
    def __reduce__(cls) -> Any: ...
    @classmethod
    def __setitem__(cls, index, object) -> Any: ...

class _vectNSt7__cxx1112basic_stringIcSt11char_traitsIcESaIcEEE:
    __instance_size__: ClassVar[int] = ...
    @classmethod
    def __init__(cls, *args, **kwargs) -> None: ...
    @classmethod
    def append(cls, *args, **kwargs) -> Any: ...
    @classmethod
    def extend(cls, *args, **kwargs) -> Any: ...
    @classmethod
    def __contains__(cls, other) -> Any: ...
    @classmethod
    def __delitem__(cls, other) -> Any: ...
    @classmethod
    def __getitem__(cls, index) -> Any: ...
    @classmethod
    def __iter__(cls) -> Any: ...
    @classmethod
    def __len__(cls) -> Any: ...
    @classmethod
    def __reduce__(cls) -> Any: ...
    @classmethod
    def __setitem__(cls, index, object) -> Any: ...

class _vectSt6vectorIdSaIdEE:
    __instance_size__: ClassVar[int] = ...
    @classmethod
    def __init__(cls, *args, **kwargs) -> None: ...
    @classmethod
    def append(cls, *args, **kwargs) -> Any: ...
    @classmethod
    def extend(cls, *args, **kwargs) -> Any: ...
    @classmethod
    def __contains__(cls, other) -> Any: ...
    @classmethod
    def __delitem__(cls, other) -> Any: ...
    @classmethod
    def __getitem__(cls, index) -> Any: ...
    @classmethod
    def __iter__(cls) -> Any: ...
    @classmethod
    def __len__(cls) -> Any: ...
    @classmethod
    def __reduce__(cls) -> Any: ...
    @classmethod
    def __setitem__(cls, index, object) -> Any: ...

class _vectSt6vectorIiSaIiEE:
    __instance_size__: ClassVar[int] = ...
    @classmethod
    def __init__(cls, *args, **kwargs) -> None: ...
    @classmethod
    def append(cls, *args, **kwargs) -> Any: ...
    @classmethod
    def extend(cls, *args, **kwargs) -> Any: ...
    @classmethod
    def __contains__(cls, other) -> Any: ...
    @classmethod
    def __delitem__(cls, other) -> Any: ...
    @classmethod
    def __getitem__(cls, index) -> Any: ...
    @classmethod
    def __iter__(cls) -> Any: ...
    @classmethod
    def __len__(cls) -> Any: ...
    @classmethod
    def __reduce__(cls) -> Any: ...
    @classmethod
    def __setitem__(cls, index, object) -> Any: ...

class _vectSt6vectorIjSaIjEE:
    __instance_size__: ClassVar[int] = ...
    @classmethod
    def __init__(cls, *args, **kwargs) -> None: ...
    @classmethod
    def append(cls, *args, **kwargs) -> Any: ...
    @classmethod
    def extend(cls, *args, **kwargs) -> Any: ...
    @classmethod
    def __contains__(cls, other) -> Any: ...
    @classmethod
    def __delitem__(cls, other) -> Any: ...
    @classmethod
    def __getitem__(cls, index) -> Any: ...
    @classmethod
    def __iter__(cls) -> Any: ...
    @classmethod
    def __len__(cls) -> Any: ...
    @classmethod
    def __reduce__(cls) -> Any: ...
    @classmethod
    def __setitem__(cls, index, object) -> Any: ...

class _vectd:
    __instance_size__: ClassVar[int] = ...
    @classmethod
    def __init__(cls, *args, **kwargs) -> None: ...
    @classmethod
    def append(cls, *args, **kwargs) -> Any: ...
    @classmethod
    def extend(cls, *args, **kwargs) -> Any: ...
    @classmethod
    def __contains__(cls, other) -> Any: ...
    @classmethod
    def __delitem__(cls, other) -> Any: ...
    @classmethod
    def __getitem__(cls, index) -> Any: ...
    @classmethod
    def __iter__(cls, boost, std) -> Any: ...
    @classmethod
    def __len__(cls) -> Any: ...
    @classmethod
    def __reduce__(cls) -> Any: ...
    @classmethod
    def __setitem__(cls, index, object) -> Any: ...

class _vecti:
    __instance_size__: ClassVar[int] = ...
    @classmethod
    def __init__(cls, *args, **kwargs) -> None: ...
    @classmethod
    def append(cls, *args, **kwargs) -> Any: ...
    @classmethod
    def extend(cls, *args, **kwargs) -> Any: ...
    @classmethod
    def __contains__(cls, other) -> Any: ...
    @classmethod
    def __delitem__(cls, other) -> Any: ...
    @classmethod
    def __getitem__(cls, index) -> Any: ...
    @classmethod
    def __iter__(cls, boost, std) -> Any: ...
    @classmethod
    def __len__(cls) -> Any: ...
    @classmethod
    def __reduce__(cls) -> Any: ...
    @classmethod
    def __setitem__(cls, index, object) -> Any: ...

class _vectj:
    __instance_size__: ClassVar[int] = ...
    @classmethod
    def __init__(cls, *args, **kwargs) -> None: ...
    @classmethod
    def append(cls, *args, **kwargs) -> Any: ...
    @classmethod
    def extend(cls, *args, **kwargs) -> Any: ...
    @classmethod
    def __contains__(cls, other) -> Any: ...
    @classmethod
    def __delitem__(cls, other) -> Any: ...
    @classmethod
    def __getitem__(cls, index) -> Any: ...
    @classmethod
    def __iter__(cls, boost, std) -> Any: ...
    @classmethod
    def __len__(cls) -> Any: ...
    @classmethod
    def __reduce__(cls) -> Any: ...
    @classmethod
    def __setitem__(cls, index, object) -> Any: ...

class ostream(std_ostream):
    @classmethod
    def __init__(cls, *args, **kwargs) -> None: ...
    @classmethod
    def __reduce__(cls) -> Any: ...

class std_ostream:
    @classmethod
    def __init__(cls, *args, **kwargs) -> None: ...
    @classmethod
    def __reduce__(cls) -> Any: ...

class streambuf:
    @classmethod
    def __init__(cls, *args, **kwargs) -> None: ...
    @classmethod
    def __reduce__(cls) -> Any: ...

def AttachFileToLog(*args, **kwargs) -> Any: ...
def DisableLog(*args, **kwargs) -> Any: ...
def EnableLog(*args, **kwargs) -> Any: ...
def LogDebugMsg(*args, **kwargs) -> Any: ...
def LogErrorMsg(*args, **kwargs) -> Any: ...
def LogInfoMsg(*args, **kwargs) -> Any: ...
def LogMessage(*args, **kwargs) -> Any: ...
def LogStatus() -> Any: ...
def LogToCppStreams() -> Any: ...
def LogToPythonLogger() -> Any: ...
def LogToPythonStderr() -> Any: ...
def LogWarningMsg(*args, **kwargs) -> Any: ...
def SeedRandomNumberGenerator(unsignedint) -> Any: ...
def WrapLogs() -> Any: ...
def _version() -> Any: ...



from _typeshed import Incomplete
from collections.abc import Generator
from rdkit.Chem.FeatMaps.FeatMapPoint import FeatMapPoint as FeatMapPoint

class FeatMapScoreMode:
    All: int
    Closest: int
    Best: int

class FeatDirScoreMode:
    Ignore: int
    DotFullRange: int
    DotPosRange: int

class FeatMapParams:
    radius: float
    width: float

    class FeatProfile:
        Gaussian: int
        Triangle: int
        Box: int

    featProfile: Incomplete

class FeatMap:
    dirScoreMode: Incomplete
    scoreMode: Incomplete
    params: Incomplete
    def __init__(
        self,
        params: Incomplete | None = ...,
        feats: Incomplete | None = ...,
        weights: Incomplete | None = ...,
    ) -> None: ...
    _feats: Incomplete
    def _initializeFeats(self, feats, weights) -> None: ...
    def AddFeature(self, feat, weight: Incomplete | None = ...) -> None: ...
    def AddFeatPoint(self, featPt) -> None: ...
    def GetFeatures(self): ...
    def GetNumFeatures(self): ...
    def GetFeature(self, i): ...
    def DropFeature(self, i) -> None: ...
    def _loopOverMatchingFeats(self, oFeat) -> Generator[Incomplete, None, None]: ...
    def GetFeatFeatScore(self, feat1, feat2, typeMatch: bool = ...): ...
    def ScoreFeats(
        self,
        featsToScore,
        mapScoreVect: Incomplete | None = ...,
        featsScoreVect: Incomplete | None = ...,
        featsToFeatMapIdx: Incomplete | None = ...,
    ): ...
    def __str__(self): ...



from typing import Any, ClassVar

BIASCHISQUARE: InfoType
BIASENTROPY: InfoType
CHISQUARE: InfoType
ENTROPY: InfoType

class BitCorrMatGenerator:
    __instance_size__: ClassVar[int] = ...
    @classmethod
    def __init__(cls, *args, **kwargs) -> None: ...
    @classmethod
    def CollectVotes(cls, RDInfoTheory, boost) -> Any: ...
    @classmethod
    def GetCorrMatrix(cls, RDInfoTheory) -> Any: ...
    @classmethod
    def SetBitList(cls, RDInfoTheory, boost) -> Any: ...
    @classmethod
    def __reduce__(cls) -> Any: ...

class InfoBitRanker:
    __instance_size__: ClassVar[int] = ...
    @classmethod
    def __init__(cls, *args, **kwargs) -> None: ...
    @classmethod
    def AccumulateVotes(cls, RDInfoTheory, boost, int) -> Any: ...
    @classmethod
    def GetTopN(cls, RDInfoTheory, int) -> Any: ...
    @classmethod
    def SetBiasList(cls, RDInfoTheory, boost) -> Any: ...
    @classmethod
    def SetMaskBits(cls, RDInfoTheory, boost) -> Any: ...
    @classmethod
    def Tester(cls, RDInfoTheory, boost) -> Any: ...
    @classmethod
    def WriteTopBitsToFile(cls, *args, **kwargs) -> Any: ...
    @classmethod
    def __reduce__(cls) -> Any: ...

class InfoType:
    BIASCHISQUARE: ClassVar[InfoType] = ...
    BIASENTROPY: ClassVar[InfoType] = ...
    CHISQUARE: ClassVar[InfoType] = ...
    ENTROPY: ClassVar[InfoType] = ...
    names: ClassVar[dict] = ...
    values: ClassVar[dict] = ...
    __slots__: ClassVar[tuple] = ...

def ChiSquare(boost) -> Any: ...
def InfoEntropy(boost) -> Any: ...
def InfoGain(boost) -> Any: ...



from _typeshed import Incomplete
from rdkit.sping.pid import Font as Font

sizedelta: int
subFraction: float
superFraction: float
greekchars: Incomplete

class StringSegment:
    super: int
    sub: int
    bold: int
    italic: int
    underline: int
    s: str
    width: int
    greek: int
    def __init__(self) -> None: ...
    def calcNewFont(self, font): ...
    def calcNewY(self, font, y): ...
    def dump(self) -> None: ...

class StringFormatter:
    bold: int
    def start_b(self, attributes) -> None: ...
    def end_b(self) -> None: ...
    italic: int
    def start_i(self, attributes) -> None: ...
    def end_i(self) -> None: ...
    underline: int
    def start_u(self, attributes) -> None: ...
    def end_u(self) -> None: ...
    super: int
    def start_super(self, attributes) -> None: ...
    def end_super(self) -> None: ...
    sub: int
    def start_sub(self, attributes) -> None: ...
    def end_sub(self) -> None: ...
    greek: int
    def start_greek(self, attributes, letter) -> None: ...
    def end_greek(self) -> None: ...
    segmentlist: Incomplete
    elements: Incomplete
    def __init__(self) -> None: ...
    def handle_data(self, data) -> None: ...
    def parseSegments(self, s): ...

def fontHeight(canvas, font: Incomplete | None = ...): ...
def fontAscent(canvas, font: Incomplete | None = ...): ...
def fontDescent(canvas, font: Incomplete | None = ...): ...
def stringWidth(canvas, s, font: Incomplete | None = ...): ...
def rotateXY(x, y, theta): ...
def drawString(
    canvas,
    s,
    x,
    y,
    font: Incomplete | None = ...,
    color: Incomplete | None = ...,
    angle: int = ...,
) -> None: ...
def test1() -> None: ...
def test2() -> None: ...
def allTagCombos(
    canvas,
    x,
    y,
    font: Incomplete | None = ...,
    color: Incomplete | None = ...,
    angle: int = ...,
): ...
def stringformatTest() -> None: ...



from _typeshed import Incomplete
from rdkit import Chem as Chem, DataStructs as DataStructs
from rdkit.Chem import (
    AllChem as AllChem,
    Draw as Draw,
    SDWriter as SDWriter,
    rdchem as rdchem,
)
from rdkit.Chem.Draw.IPythonConsole import InteractiveRenderer as InteractiveRenderer
from rdkit.Chem.Scaffolds import MurckoScaffold as MurckoScaffold

log: Incomplete

def _getPandasVersion(): ...

pandasVersion: Incomplete
defPandasRendering: Incomplete
defPandasRepr: Incomplete
highlightSubstructures: bool
molRepresentation: str
molSize: Incomplete

def getAdjustmentAttr(): ...
def _patched_HTMLFormatter_write_cell(self, s, *args, **kwargs): ...
def _patched_get_adjustment(): ...
def patchPandasrepr(self, **kwargs): ...
def patchPandasHTMLrepr(self, **kwargs): ...
def is_molecule_image(s): ...

class RenderMoleculeAdjustment:
    inner_adjustment: Incomplete
    def __init__(self, inner_adjustment) -> None: ...
    def len(self, text): ...
    def justify(self, texts, max_len, mode: str = ...): ...
    def adjoin(self, space, *lists, **kwargs): ...

def _get_image(x): ...
def _fingerprinter(x, y): ...
def _molge(x, y): ...
def PrintAsBase64PNGString(x, renderer: Incomplete | None = ...): ...
def PrintDefaultMolRep(x): ...
def _MolPlusFingerprint(m): ...
def RenderImagesInAllDataFrames(images: bool = ...) -> None: ...
def AddMoleculeColumnToFrame(
    frame, smilesCol: str = ..., molCol: str = ..., includeFingerprints: bool = ...
): ...
def ChangeMoleculeRendering(
    frame: Incomplete | None = ..., renderer: str = ...
) -> None: ...
def LoadSDF(
    filename,
    idName: str = ...,
    molColName: str = ...,
    includeFingerprints: bool = ...,
    isomericSmiles: bool = ...,
    smilesName: Incomplete | None = ...,
    embedProps: bool = ...,
    removeHs: bool = ...,
    strictParsing: bool = ...,
): ...
def WriteSDF(
    df,
    out,
    molColName: str = ...,
    idName: Incomplete | None = ...,
    properties: Incomplete | None = ...,
    allNumeric: bool = ...,
) -> None: ...

_saltRemover: Incomplete

def RemoveSaltsFromFrame(frame, molCol: str = ...): ...
def SaveSMILESFromFrame(
    frame, outFile, molCol: str = ..., NamesCol: str = ..., isomericSmiles: bool = ...
) -> None: ...
def SaveXlsxFromFrame(frame, outFile, molCol: str = ..., size=...) -> None: ...
def FrameToGridImage(
    frame, column: str = ..., legendsCol: Incomplete | None = ..., **kwargs
): ...
def AddMurckoToFrame(
    frame, molCol: str = ..., MurckoCol: str = ..., Generic: bool = ...
): ...
def AlignMol(mol, scaffold): ...
def AlignToScaffold(frame, molCol: str = ..., scaffoldCol: str = ...): ...
def RGroupDecompositionToFrame(
    groups, mols, include_core: bool = ..., redraw_sidechains: bool = ...
): ...
def InstallPandasTools() -> None: ...
def UninstallPandasTools() -> None: ...

_originalSettings: Incomplete

def _runDoctests(verbose: Incomplete | None = ...) -> None: ...



from _typeshed import Incomplete
from rdkit import RDConfig as RDConfig

def SmilesToGif(
    smiles,
    fileNames,
    size=...,
    cmd: Incomplete | None = ...,
    dblSize: int = ...,
    frame: int = ...,
): ...
def SmilesToImage(smiles, **kwargs): ...



from _typeshed import Incomplete
from rdkit import Chem as Chem

def cmp(t1, t2): ...

periodicTable: Incomplete

class Font:
    face: Incomplete
    size: Incomplete
    weight: Incomplete
    name: Incomplete
    def __init__(
        self,
        face: Incomplete | None = ...,
        size: Incomplete | None = ...,
        name: Incomplete | None = ...,
        weight: Incomplete | None = ...,
    ) -> None: ...

class DrawingOptions:
    dotsPerAngstrom: int
    useFraction: float
    atomLabelFontFace: str
    atomLabelFontSize: int
    atomLabelMinFontSize: int
    atomLabelDeuteriumTritium: bool
    bondLineWidth: float
    dblBondOffset: float
    dblBondLengthFrac: float
    defaultColor: Incomplete
    selectColor: Incomplete
    bgColor: Incomplete
    colorBonds: bool
    noCarbonSymbols: bool
    includeAtomNumbers: bool
    atomNumberOffset: int
    radicalSymbol: str
    dash: Incomplete
    wedgeDashedBonds: bool
    showUnknownDoubleBonds: bool
    coordScale: float
    elemDict: Incomplete

class MolDrawing:
    canvas: Incomplete
    canvasSize: Incomplete
    drawingOptions: Incomplete
    atomPs: Incomplete
    boundingBoxes: Incomplete
    def __init__(
        self, canvas: Incomplete | None = ..., drawingOptions: Incomplete | None = ...
    ) -> None: ...
    def transformPoint(self, pos): ...
    def _getBondOffset(self, p1, p2): ...
    def _getOffsetBondPts(
        self, p1, p2, offsetX, offsetY, lenFrac: Incomplete | None = ...
    ): ...
    def _offsetDblBond(
        self,
        p1,
        p2,
        bond,
        a1,
        a2,
        conf,
        direction: int = ...,
        lenFrac: Incomplete | None = ...,
    ): ...
    def _getBondAttachmentCoordinates(self, p1, p2, labelSize): ...
    def _drawWedgedBond(
        self,
        bond,
        pos,
        nbrPos,
        width: Incomplete | None = ...,
        color: Incomplete | None = ...,
        dash: Incomplete | None = ...,
    ) -> None: ...
    def _drawBond(
        self,
        bond,
        atom,
        nbr,
        pos,
        nbrPos,
        conf,
        width: Incomplete | None = ...,
        color: Incomplete | None = ...,
        color2: Incomplete | None = ...,
        labelSize1: Incomplete | None = ...,
        labelSize2: Incomplete | None = ...,
    ) -> None: ...
    molTrans: Incomplete
    currAtomLabelFontSize: Incomplete
    drawingTrans: Incomplete
    def scaleAndCenter(
        self,
        mol,
        conf,
        coordCenter: bool = ...,
        canvasSize: Incomplete | None = ...,
        ignoreHs: bool = ...,
    ) -> None: ...
    def _drawLabel(
        self, label, pos, baseOffset, font, color: Incomplete | None = ..., **kwargs
    ): ...
    currDotsPerAngstrom: Incomplete
    activeMol: Incomplete
    bondRings: Incomplete
    def AddMol(
        self,
        mol,
        centerIt: bool = ...,
        molTrans: Incomplete | None = ...,
        drawingTrans: Incomplete | None = ...,
        highlightAtoms=...,
        confId: int = ...,
        flagCloseContactsDist: int = ...,
        highlightMap: Incomplete | None = ...,
        ignoreHs: bool = ...,
        highlightBonds=...,
        **kwargs
    ) -> None: ...



from rdkit.sping.pid import *
from math import *
from _typeshed import Incomplete
from rdkit.sping.PDF import pdfmetrics as pdfmetrics, pidPDF as pidPDF

def colorToRL(color): ...

class RLCanvas(Canvas):
    size: Incomplete
    drawing: Incomplete
    def __init__(self, size=..., name: str = ...) -> None: ...
    def _initOutput(self) -> None: ...
    def clear(self) -> None: ...
    def flush(self) -> None: ...
    def save(
        self, file: Incomplete | None = ..., format: Incomplete | None = ...
    ) -> None: ...
    def fixY(self, y): ...
    def _findPostScriptFontName(self, font): ...
    def drawLine(
        self,
        x1,
        y1,
        x2,
        y2,
        color: Incomplete | None = ...,
        width: Incomplete | None = ...,
        dash: Incomplete | None = ...,
        **kwargs
    ) -> None: ...
    def drawArc(
        self,
        x1,
        y1,
        x2,
        y2,
        startAng: int = ...,
        extent: int = ...,
        edgeColor: Incomplete | None = ...,
        edgeWidth: Incomplete | None = ...,
        fillColor: Incomplete | None = ...,
        dash: Incomplete | None = ...,
        **kwargs
    ) -> None: ...
    def drawPolygon(
        self,
        pointlist,
        edgeColor: Incomplete | None = ...,
        edgeWidth: Incomplete | None = ...,
        fillColor=...,
        closed: int = ...,
        dash: Incomplete | None = ...,
        **kwargs
    ) -> None: ...
    def drawString(
        self,
        s,
        x,
        y,
        font: Incomplete | None = ...,
        color: Incomplete | None = ...,
        angle: int = ...,
        **kwargs
    ) -> None: ...
    def drawImage(
        self,
        image,
        x1,
        y1,
        x2: Incomplete | None = ...,
        y2: Incomplete | None = ...,
        **kwargs
    ) -> None: ...
    def stringWidth(self, s, font: Incomplete | None = ...): ...
    def fontAscent(self, font: Incomplete | None = ...): ...
    def fontDescent(self, font: Incomplete | None = ...): ...

def test(): ...
def dashtest(): ...
def testit(canvas, s, x, y, font: Incomplete | None = ...) -> None: ...
def test2() -> None: ...



from typing import Any

class ForceField:
    @classmethod
    def __init__(cls, *args, **kwargs) -> None: ...
    @classmethod
    def AddDistanceConstraint(cls, *args, **kwargs) -> Any: ...
    @classmethod
    def AddExtraPoint(cls, *args, **kwargs) -> Any: ...
    @classmethod
    def AddFixedPoint(cls, ForceFields, unsignedint) -> Any: ...
    @classmethod
    def CalcEnergy(cls, ForceFields) -> Any: ...
    @classmethod
    def CalcGrad(cls, ForceFields) -> Any: ...
    @classmethod
    def Dimension(cls, ForceFields) -> Any: ...
    @classmethod
    def GetExtraPointPos(cls, ForceFields, unsignedint) -> Any: ...
    @classmethod
    def Initialize(cls, ForceFields) -> Any: ...
    @classmethod
    def MMFFAddAngleConstraint(cls, *args, **kwargs) -> Any: ...
    @classmethod
    def MMFFAddDistanceConstraint(cls, *args, **kwargs) -> Any: ...
    @classmethod
    def MMFFAddPositionConstraint(cls, *args, **kwargs) -> Any: ...
    @classmethod
    def MMFFAddTorsionConstraint(cls, *args, **kwargs) -> Any: ...
    @classmethod
    def Minimize(cls, ForceFields) -> Any: ...
    @classmethod
    def MinimizeTrajectory(cls, *args, **kwargs) -> Any: ...
    @classmethod
    def NumPoints(cls, ForceFields) -> Any: ...
    @classmethod
    def Positions(cls, ForceFields) -> Any: ...
    @classmethod
    def UFFAddAngleConstraint(cls, *args, **kwargs) -> Any: ...
    @classmethod
    def UFFAddDistanceConstraint(cls, *args, **kwargs) -> Any: ...
    @classmethod
    def UFFAddPositionConstraint(cls, *args, **kwargs) -> Any: ...
    @classmethod
    def UFFAddTorsionConstraint(cls, *args, **kwargs) -> Any: ...
    @classmethod
    def __reduce__(cls) -> Any: ...

class MMFFMolProperties:
    @classmethod
    def __init__(cls, *args, **kwargs) -> None: ...
    @classmethod
    def GetMMFFAngleBendParams(cls, *args, **kwargs) -> Any: ...
    @classmethod
    def GetMMFFAtomType(cls, ForceFields, unsignedint) -> Any: ...
    @classmethod
    def GetMMFFBondStretchParams(cls, *args, **kwargs) -> Any: ...
    @classmethod
    def GetMMFFFormalCharge(cls, ForceFields, unsignedint) -> Any: ...
    @classmethod
    def GetMMFFOopBendParams(cls, *args, **kwargs) -> Any: ...
    @classmethod
    def GetMMFFPartialCharge(cls, ForceFields, unsignedint) -> Any: ...
    @classmethod
    def GetMMFFStretchBendParams(cls, *args, **kwargs) -> Any: ...
    @classmethod
    def GetMMFFTorsionParams(cls, *args, **kwargs) -> Any: ...
    @classmethod
    def GetMMFFVdWParams(cls, *args, **kwargs) -> Any: ...
    @classmethod
    def SetMMFFAngleTerm(cls, ForceFields) -> Any: ...
    @classmethod
    def SetMMFFBondTerm(cls, ForceFields) -> Any: ...
    @classmethod
    def SetMMFFDielectricConstant(cls, ForceFields) -> Any: ...
    @classmethod
    def SetMMFFDielectricModel(cls, ForceFields) -> Any: ...
    @classmethod
    def SetMMFFEleTerm(cls, ForceFields) -> Any: ...
    @classmethod
    def SetMMFFOopTerm(cls, ForceFields) -> Any: ...
    @classmethod
    def SetMMFFStretchBendTerm(cls, ForceFields) -> Any: ...
    @classmethod
    def SetMMFFTorsionTerm(cls, ForceFields) -> Any: ...
    @classmethod
    def SetMMFFVariant(cls, ForceFields) -> Any: ...
    @classmethod
    def SetMMFFVdWTerm(cls, ForceFields) -> Any: ...
    @classmethod
    def SetMMFFVerbosity(cls, ForceFields) -> Any: ...
    @classmethod
    def __reduce__(cls) -> Any: ...



from _typeshed import Incomplete
from rdkit import Chem as Chem
from rdkit.Chem.Pharm2D import Utils as Utils

class MatchError(Exception): ...

_verbose: int

def GetAtomsMatchingBit(
    sigFactory,
    bitIdx,
    mol,
    dMat: Incomplete | None = ...,
    justOne: int = ...,
    matchingAtoms: Incomplete | None = ...,
): ...
def _exampleCode() -> None: ...



from typing import Any, ClassVar

class Point2D:
    __instance_size__: ClassVar[int] = ...
    __safe_for_unpickling__: ClassVar[bool] = ...
    x: Any
    y: Any
    @classmethod
    def __init__(cls, *args, **kwargs) -> None: ...
    @classmethod
    def AngleTo(cls, *args, **kwargs) -> Any: ...
    @classmethod
    def DirectionVector(cls, *args, **kwargs) -> Any: ...
    @classmethod
    def DotProduct(cls, *args, **kwargs) -> Any: ...
    @classmethod
    def Length(cls, RDGeom) -> Any: ...
    @classmethod
    def LengthSq(cls, RDGeom) -> Any: ...
    @classmethod
    def Normalize(cls, RDGeom) -> Any: ...
    @classmethod
    def SignedAngleTo(cls, *args, **kwargs) -> Any: ...
    @classmethod
    def __add__(cls, other) -> Any: ...
    @classmethod
    def __getinitargs__(cls, RDGeom) -> Any: ...
    @classmethod
    def __getitem__(cls, RDGeom, int) -> Any: ...
    @classmethod
    def __iadd__(cls, boost, RDGeom) -> Any: ...
    @classmethod
    def __idiv__(cls, RDGeom, double) -> Any: ...
    @classmethod
    def __imul__(cls, RDGeom, double) -> Any: ...
    @classmethod
    def __isub__(cls, boost, RDGeom) -> Any: ...
    @classmethod
    def __len__(cls, RDGeom) -> Any: ...
    @classmethod
    def __mul__(cls, RDGeom, double) -> Any: ...
    @classmethod
    def __reduce__(cls) -> Any: ...
    @classmethod
    def __sub__(cls, other) -> Any: ...
    @classmethod
    def __truediv__(cls, RDGeom, double) -> Any: ...

class Point3D:
    __instance_size__: ClassVar[int] = ...
    __safe_for_unpickling__: ClassVar[bool] = ...
    x: Any
    y: Any
    z: Any
    @classmethod
    def __init__(cls, *args, **kwargs) -> None: ...
    @classmethod
    def AngleTo(cls, *args, **kwargs) -> Any: ...
    @classmethod
    def CrossProduct(cls, *args, **kwargs) -> Any: ...
    @classmethod
    def DirectionVector(cls, *args, **kwargs) -> Any: ...
    @classmethod
    def Distance(cls, *args, **kwargs) -> Any: ...
    @classmethod
    def DotProduct(cls, *args, **kwargs) -> Any: ...
    @classmethod
    def Length(cls, RDGeom) -> Any: ...
    @classmethod
    def LengthSq(cls, RDGeom) -> Any: ...
    @classmethod
    def Normalize(cls, RDGeom) -> Any: ...
    @classmethod
    def SignedAngleTo(cls, *args, **kwargs) -> Any: ...
    @classmethod
    def __add__(cls, other) -> Any: ...
    @classmethod
    def __getinitargs__(cls, RDGeom) -> Any: ...
    @classmethod
    def __getitem__(cls, RDGeom, int) -> Any: ...
    @classmethod
    def __iadd__(cls, boost, RDGeom) -> Any: ...
    @classmethod
    def __idiv__(cls, RDGeom, double) -> Any: ...
    @classmethod
    def __imul__(cls, RDGeom, double) -> Any: ...
    @classmethod
    def __isub__(cls, boost, RDGeom) -> Any: ...
    @classmethod
    def __len__(cls, RDGeom) -> Any: ...
    @classmethod
    def __mul__(cls, RDGeom, double) -> Any: ...
    @classmethod
    def __reduce__(cls) -> Any: ...
    @classmethod
    def __sub__(cls, other) -> Any: ...
    @classmethod
    def __truediv__(cls, RDGeom, double) -> Any: ...

class PointND:
    __instance_size__: ClassVar[int] = ...
    __safe_for_unpickling__: ClassVar[bool] = ...
    @classmethod
    def __init__(cls, *args, **kwargs) -> None: ...
    @classmethod
    def AngleTo(cls, *args, **kwargs) -> Any: ...
    @classmethod
    def DirectionVector(cls, *args, **kwargs) -> Any: ...
    @classmethod
    def Distance(cls, *args, **kwargs) -> Any: ...
    @classmethod
    def DotProduct(cls, *args, **kwargs) -> Any: ...
    @classmethod
    def Length(cls, RDGeom) -> Any: ...
    @classmethod
    def LengthSq(cls, RDGeom) -> Any: ...
    @classmethod
    def Normalize(cls, RDGeom) -> Any: ...
    @classmethod
    def __add__(cls, other) -> Any: ...
    @classmethod
    def __getinitargs__(cls, RDGeom) -> Any: ...
    @classmethod
    def __getitem__(cls, RDGeom, int) -> Any: ...
    def __getstate__(self) -> Any: ...
    @classmethod
    def __iadd__(cls, boost, RDGeom) -> Any: ...
    @classmethod
    def __idiv__(cls, RDGeom, double) -> Any: ...
    @classmethod
    def __imul__(cls, RDGeom, double) -> Any: ...
    @classmethod
    def __isub__(cls, boost, RDGeom) -> Any: ...
    @classmethod
    def __len__(cls, RDGeom) -> Any: ...
    @classmethod
    def __mul__(cls, RDGeom, double) -> Any: ...
    @classmethod
    def __reduce__(cls) -> Any: ...
    @classmethod
    def __setitem__(cls, RDGeom, int, double) -> Any: ...
    @classmethod
    def __setstate__(cls, RDGeom, boost) -> Any: ...
    @classmethod
    def __sub__(cls, other) -> Any: ...
    @classmethod
    def __truediv__(cls, RDGeom, double) -> Any: ...

class UniformGrid3D_:
    __instance_size__: ClassVar[int] = ...
    __safe_for_unpickling__: ClassVar[bool] = ...
    @classmethod
    def __init__(cls, *args, **kwargs) -> None: ...
    @classmethod
    def CompareParams(cls, *args, **kwargs) -> Any: ...
    @classmethod
    def GetGridIndex(cls, *args, **kwargs) -> Any: ...
    @classmethod
    def GetGridIndices(cls, RDGeom, unsignedint) -> Any: ...
    @classmethod
    def GetGridPointIndex(cls, *args, **kwargs) -> Any: ...
    @classmethod
    def GetGridPointLoc(cls, RDGeom, unsignedint) -> Any: ...
    @classmethod
    def GetNumX(cls, RDGeom) -> Any: ...
    @classmethod
    def GetNumY(cls, RDGeom) -> Any: ...
    @classmethod
    def GetNumZ(cls, RDGeom) -> Any: ...
    @classmethod
    def GetOccupancyVect(cls, RDGeom) -> Any: ...
    @classmethod
    def GetOffset(cls, RDGeom) -> Any: ...
    @classmethod
    def GetSize(cls, RDGeom) -> Any: ...
    @classmethod
    def GetSpacing(cls, RDGeom) -> Any: ...
    @classmethod
    def GetVal(cls, RDGeom, unsignedint) -> Any: ...
    @classmethod
    def GetValPoint(cls, *args, **kwargs) -> Any: ...
    @classmethod
    def SetSphereOccupancy(cls, *args, **kwargs) -> Any: ...
    @classmethod
    def SetVal(cls, *args, **kwargs) -> Any: ...
    @classmethod
    def SetValPoint(cls, *args, **kwargs) -> Any: ...
    @classmethod
    def __getinitargs__(cls, RDGeom) -> Any: ...
    @classmethod
    def __iadd__(cls, boost, RDGeom) -> Any: ...
    @classmethod
    def __iand__(cls, boost, RDGeom) -> Any: ...
    @classmethod
    def __ior__(cls, boost, RDGeom) -> Any: ...
    @classmethod
    def __isub__(cls, boost, RDGeom) -> Any: ...
    @classmethod
    def __reduce__(cls) -> Any: ...

def ComputeDihedralAngle(*args, **kwargs) -> Any: ...
def ComputeGridCentroid(*args, **kwargs) -> Any: ...
def ComputeSignedDihedralAngle(*args, **kwargs) -> Any: ...
def FindGridTerminalPoints(*args, **kwargs) -> Any: ...
def ProtrudeDistance(*args, **kwargs) -> Any: ...
def TanimotoDistance(*args, **kwargs) -> Any: ...
def TverskyIndex(*args, **kwargs) -> Any: ...
def UniformGrid3D(*args, **kwargs) -> Any: ...
def WriteGridToFile(*args, **kwargs) -> Any: ...



from _typeshed import Incomplete
from rdkit import Chem as Chem, DataStructs as DataStructs
from rdkit.Chem import rdMolDescriptors as rdMolDescriptors

smartsPatts: Incomplete
maccsKeys: Incomplete

def _InitKeys(keyList, keyDict) -> None: ...
def _pyGenMACCSKeys(mol, **kwargs): ...

GenMACCSKeys: Incomplete
FingerprintMol: Incomplete

def _test(): ...



from _typeshed import Incomplete
from rdkit import RDConfig as RDConfig

sqlTextTypes: Incomplete
sqlIntTypes: Incomplete
sqlFloatTypes: Incomplete
sqlBinTypes: Incomplete
getTablesSql: str
getTablesAndViewsSql: str
getDbSql: str
fileWildcard: Incomplete
placeHolder: str
binaryTypeName: str
RDTestDatabase: str
dbFileWildcard: str
fileWildcard = dbFileWildcard
binaryHolder = memoryview

def connect(x, *args): ...



from typing import Optional
import numpy.typing as npt
from rdkit.Chem import *
from rdkit.Chem.ChemicalFeatures import *
from rdkit.Chem.rdChemReactions import (
    CartesianProductStrategy as CartesianProductStrategy,
    ChemicalReaction as ChemicalReaction,
    EnumerateLibrary as EnumerateLibrary,
    EnumerateLibraryBase as EnumerateLibraryBase,
    EnumerationParams as EnumerationParams,
    EnumerationStrategyBase as EnumerationStrategyBase,
    EvenSamplePairsStrategy as EvenSamplePairsStrategy,
    FingerprintType as FingerprintType,
    MOL_SPTR_VECT as MOL_SPTR_VECT,
    RandomSampleAllBBsStrategy as RandomSampleAllBBsStrategy,
    RandomSampleStrategy as RandomSampleStrategy,
    ReactionFingerprintParams as ReactionFingerprintParams,
    VectMolVect as VectMolVect,
    VectSizeT as VectSizeT,
    VectorOfStringVectors as VectorOfStringVectors,
    Compute2DCoordsForReaction as Compute2DCoordsForReaction,
    CreateDifferenceFingerprintForReaction as CreateDifferenceFingerprintForReaction,
    CreateStructuralFingerprintForReaction as CreateStructuralFingerprintForReaction,
    EnumerateLibraryCanSerialize as EnumerateLibraryCanSerialize,
    GetChemDrawRxnAdjustParams as GetChemDrawRxnAdjustParams,
    GetDefaultAdjustParams as GetDefaultAdjustParams,
    HasAgentTemplateSubstructMatch as HasAgentTemplateSubstructMatch,
    HasProductTemplateSubstructMatch as HasProductTemplateSubstructMatch,
    HasReactantTemplateSubstructMatch as HasReactantTemplateSubstructMatch,
    HasReactionAtomMapping as HasReactionAtomMapping,
    HasReactionSubstructMatch as HasReactionSubstructMatch,
    IsReactionTemplateMoleculeAgent as IsReactionTemplateMoleculeAgent,
    MatchOnlyAtRgroupsAdjustParams as MatchOnlyAtRgroupsAdjustParams,
    PreprocessReaction as PreprocessReaction,
    ReactionFromMolecule as ReactionFromMolecule,
    ReactionFromPNGFile as ReactionFromPNGFile,
    ReactionFromPNGString as ReactionFromPNGString,
    ReactionFromRxnBlock as ReactionFromRxnBlock,
    ReactionFromRxnFile as ReactionFromRxnFile,
    ReactionFromSmarts as ReactionFromSmarts,
    ReactionMetadataToPNGFile as ReactionMetadataToPNGFile,
    ReactionMetadataToPNGString as ReactionMetadataToPNGString,
    ReactionToMolecule as ReactionToMolecule,
    ReactionToRxnBlock as ReactionToRxnBlock,
    ReactionToSmarts as ReactionToSmarts,
    ReactionToSmiles as ReactionToSmiles,
    ReactionToV3KRxnBlock as ReactionToV3KRxnBlock,
    ReduceProductToSideChains as ReduceProductToSideChains,
    RemoveMappingNumbersFromReactions as RemoveMappingNumbersFromReactions,
    SanitizeRxn as SanitizeRxn,
    UpdateProductsStereochemistry as UpdateProductsStereochemistry,
)
from rdkit.Chem.rdDepictor import *
from rdkit.Chem.rdDistGeom import *
from rdkit.Chem.rdForceFieldHelpers import *
from rdkit.Chem.rdMolAlign import *
from rdkit.Chem.rdMolDescriptors import *
from rdkit.Chem.rdMolTransforms import *
from rdkit.Chem.rdPartialCharges import *
from rdkit.Chem.rdReducedGraphs import *
from rdkit.Chem.rdShapeHelpers import *
from rdkit.Chem.rdqueries import *
from rdkit.Chem.rdMolEnumerator import *
from rdkit.Chem.rdSLNParse import *
from collections.abc import Generator
from rdkit import (
    DataStructs as DataStructs,
    ForceField as ForceField,
    RDConfig as RDConfig,
    rdBase as rdBase,
)
from rdkit.Chem.EnumerateStereoisomers import (
    EnumerateStereoisomers as EnumerateStereoisomers,
    StereoEnumerationOptions as StereoEnumerationOptions,
)
from rdkit.Geometry import rdGeometry as rdGeometry
from rdkit.RDLogger import logger as logger

def TransformMol(mol: Mol, tform, confId: int = -1, keepConfs: bool = False) -> None:
    """
    Applies the transformation (usually a 4x4 double matrix) to a molecule
    if keepConfs is False then all but that conformer are removed
    """

def ComputeMolShape(
    mol: Mol, confId: int = -1, boxDim=(20, 20, 20), spacing: float = 0.5, **kwargs
):
    """returns a grid representation of the molecule's shape"""

def ComputeMolVolume(
    mol: Mol, confId: int = -1, gridSpacing: float = 0.2, boxMargin: float = 2.0
) -> float:
    """
    Calculates the volume of a particular conformer of a molecule
    based on a grid-encoding of the molecular shape.

    A bit of demo as well as a test of github #1883:

    >>> from rdkit import Chem
    >>> from rdkit.Chem import AllChem
    >>> mol = Chem.AddHs(Chem.MolFromSmiles('C'))
    >>> AllChem.EmbedMolecule(mol)
    0
    >>> ComputeMolVolume(mol)
    28...
    >>> mol = Chem.AddHs(Chem.MolFromSmiles('O'))
    >>> AllChem.EmbedMolecule(mol)
    0
    >>> ComputeMolVolume(mol)
    20...
    """

def GetConformerRMS(
    mol: Mol,
    confId1: int,
    confId2: int,
    atomIds: Optional[npt.ArrayLike] = None,
    prealigned: bool = False,
) -> float:
    """
    Returns the RMS between two conformations.
    By default, the conformers will be aligned to the first conformer
    before the RMS calculation and, as a side-effect, the second will be left
    in the aligned state.

    Arguments:
    - mol:        the molecule
    - confId1:    the id of the first conformer
    - confId2:    the id of the second conformer
    - atomIds:    (optional) list of atom ids to use a points for
                    alingment - defaults to all atoms
    - prealigned: (optional) by default the conformers are assumed
                    be unaligned and the second conformer be aligned
                    to the first
    """

def GetConformerRMSMatrix(
    mol: Mol, atomIds: Optional[npt.ArrayLike] = None, prealigned: bool = False
) -> list[float]:
    """
    Returns the RMS matrix of the conformers of a molecule.
    As a side-effect, the conformers will be aligned to the first
    conformer (i.e. the reference) and will left in the aligned state.

    Arguments:
      - mol:     the molecule
      - atomIds: (optional) list of atom ids to use a points for
                 alingment - defaults to all atoms
      - prealigned: (optional) by default the conformers are assumed
                    be unaligned and will therefore be aligned to the
                    first conformer

    Note that the returned RMS matrix is symmetrical, i.e. it is the
    lower half of the matrix, e.g. for 5 conformers::

      rmsmatrix = [ a,
                    b, c,
                    d, e, f,
                    g, h, i, j]

    where a is the RMS between conformers 0 and 1, b is the RMS between
    conformers 0 and 2, etc.
    This way it can be directly used as distance matrix in e.g. Butina
    clustering.
    """

def EnumerateLibraryFromReaction(
    reaction: ChemicalReaction,
    sidechainSets: list[list[Mol]],
    returnReactants: bool = False,
) -> Generator[tuple[Mol, ...], None, None]:
    """
    Returns a generator for the virtual library defined by
    a reaction and a sequence of sidechain sets

    >>> from rdkit import Chem
    >>> from rdkit.Chem import AllChem
    >>> s1=[Chem.MolFromSmiles(x) for x in ('NC','NCC')]
    >>> s2=[Chem.MolFromSmiles(x) for x in ('OC=O','OC(=O)C')]
    >>> rxn = AllChem.ReactionFromSmarts('[O:2]=[C:1][OH].[N:3]>>[O:2]=[C:1][N:3]')
    >>> r = AllChem.EnumerateLibraryFromReaction(rxn,[s2,s1])
    >>> [Chem.MolToSmiles(x[0]) for x in list(r)]
    ['CNC=O', 'CCNC=O', 'CNC(C)=O', 'CCNC(C)=O']

    Note that this is all done in a lazy manner, so "infinitely" large libraries can
    be done without worrying about running out of memory. Your patience will run out first:

    Define a set of 10000 amines:

    >>> amines = (Chem.MolFromSmiles('N'+'C'*x) for x in range(10000))

    ... a set of 10000 acids

    >>> acids = (Chem.MolFromSmiles('OC(=O)'+'C'*x) for x in range(10000))

    ... now the virtual library (1e8 compounds in principle):

    >>> r = AllChem.EnumerateLibraryFromReaction(rxn,[acids,amines])

    ... look at the first 4 compounds:

    >>> [Chem.MolToSmiles(next(r)[0]) for x in range(4)]
    ['NC=O', 'CNC=O', 'CCNC=O', 'CCCNC=O']
    """

def ConstrainedEmbed(
    mol: Mol,
    core: Mol,
    useTethers: bool = True,
    coreConfId: int = -1,
    randomseed: int = 2342,
    getForceField=UFFGetMoleculeForceField,
    **kwargs
) -> Mol:
    """
    generates an embedding of a molecule where part of the molecule
    is constrained to have particular coordinates

    Arguments
      - mol: the molecule to embed
      - core: the molecule to use as a source of constraints
      - useTethers: (optional) if True, the final conformation will be
          optimized subject to a series of extra forces that pull the
          matching atoms to the positions of the core atoms. Otherwise
          simple distance constraints based on the core atoms will be
          used in the optimization.
      - coreConfId: (optional) id of the core conformation to use
      - randomSeed: (optional) seed for the random number generator


    An example, start by generating a template with a 3D structure:

    >>> from rdkit.Chem import AllChem
    >>> template = AllChem.MolFromSmiles("c1nn(Cc2ccccc2)cc1")
    >>> AllChem.EmbedMolecule(template)
    0
    >>> AllChem.UFFOptimizeMolecule(template)
    0

    Here's a molecule:

    >>> mol = AllChem.MolFromSmiles("c1nn(Cc2ccccc2)cc1-c3ccccc3")

    Now do the constrained embedding

    >>> mol = AllChem.ConstrainedEmbed(mol, template)

    Demonstrate that the positions are nearly the same with template:

    >>> import math
    >>> molp = mol.GetConformer().GetAtomPosition(0)
    >>> templatep = template.GetConformer().GetAtomPosition(0)
    >>> all(math.isclose(v, 0.0, abs_tol=0.01) for v in molp-templatep)
    True
    >>> molp = mol.GetConformer().GetAtomPosition(1)
    >>> templatep = template.GetConformer().GetAtomPosition(1)
    >>> all(math.isclose(v, 0.0, abs_tol=0.01) for v in molp-templatep)
    True

    """

def AssignBondOrdersFromTemplate(refmol: Mol, mol: Mol) -> Mol:
    """
    assigns bond orders to a molecule based on the
    bond orders in a template molecule

    Arguments
      - refmol: the template molecule
      - mol: the molecule to assign bond orders to

    An example, start by generating a template from a SMILES
    and read in the PDB structure of the molecule

    >>> import os
    >>> from rdkit.Chem import AllChem
    >>> template = AllChem.MolFromSmiles("CN1C(=NC(C1=O)(c2ccccc2)c3ccccc3)N")
    >>> mol = AllChem.MolFromPDBFile(os.path.join(RDConfig.RDCodeDir, 'Chem', 'test_data', '4DJU_lig.pdb'))
    >>> len([1 for b in template.GetBonds() if b.GetBondTypeAsDouble() == 1.0])
    8
    >>> len([1 for b in mol.GetBonds() if b.GetBondTypeAsDouble() == 1.0])
    22

    Now assign the bond orders based on the template molecule

    >>> newMol = AllChem.AssignBondOrdersFromTemplate(template, mol)
    >>> len([1 for b in newMol.GetBonds() if b.GetBondTypeAsDouble() == 1.0])
    8

    Note that the template molecule should have no explicit hydrogens
    else the algorithm will fail.

    It also works if there are different formal charges (this was github issue 235):

    >>> template=AllChem.MolFromSmiles('CN(C)C(=O)Cc1ccc2c(c1)NC(=O)c3ccc(cc3N2)c4ccc(c(c4)OC)[N+](=O)[O-]')
    >>> mol = AllChem.MolFromMolFile(os.path.join(RDConfig.RDCodeDir, 'Chem', 'test_data', '4FTR_lig.mol'))
    >>> AllChem.MolToSmiles(mol)
    'COC1CC(C2CCC3C(O)NC4CC(CC(O)N(C)C)CCC4NC3C2)CCC1N(O)O'
    >>> newMol = AllChem.AssignBondOrdersFromTemplate(template, mol)
    >>> AllChem.MolToSmiles(newMol)
    'COc1cc(-c2ccc3c(c2)Nc2ccc(CC(=O)N(C)C)cc2NC3=O)ccc1[N+](=O)[O-]'

    """

def _runDoctests(verbose=None) -> None: ...



from _typeshed import Incomplete

class BitEnsemble:
    _bits: Incomplete
    def __init__(self, bits: Incomplete | None = ...) -> None: ...
    def SetBits(self, bits) -> None: ...
    def AddBit(self, bit) -> None: ...
    def GetBits(self): ...
    def GetNumBits(self): ...
    def ScoreWithOnBits(self, other): ...
    def ScoreWithIndex(self, other): ...



from _typeshed import Incomplete
from rdkit import RDConfig as RDConfig
from rdkit.Dbase import DbModule as DbModule

def ValidateRDId(ID): ...
def RDIdToInt(ID, validate: int = ...): ...
def IndexToRDId(idx, leadText: str = ...): ...
def GetNextId(conn, table, idColName: str = ...): ...
def GetNextRDId(conn, table, idColName: str = ..., leadText: str = ...): ...
def RegisterItem(
    conn,
    table,
    value,
    columnName,
    data: Incomplete | None = ...,
    id: str = ...,
    idColName: str = ...,
    leadText: str = ...,
): ...
def RegisterItems(
    conn,
    table,
    values,
    columnName,
    rows,
    startId: str = ...,
    idColName: str = ...,
    leadText: str = ...,
): ...

_roundtripTests: str
__test__: Incomplete

def _test(): ...



from typing import Any, ClassVar

avalonSSSBits: int
avalonSimilarityBits: int

class StruChkFlag:
    alias_conversion_failed: ClassVar[StruChkFlag] = ...
    atom_check_failed: ClassVar[StruChkFlag] = ...
    atom_clash: ClassVar[StruChkFlag] = ...
    bad_molecule: ClassVar[StruChkFlag] = ...
    dubious_stereo_removed: ClassVar[StruChkFlag] = ...
    either_warning: ClassVar[StruChkFlag] = ...
    fragments_found: ClassVar[StruChkFlag] = ...
    names: ClassVar[dict] = ...
    recharged: ClassVar[StruChkFlag] = ...
    size_check_failed: ClassVar[StruChkFlag] = ...
    stereo_error: ClassVar[StruChkFlag] = ...
    stereo_forced_bad: ClassVar[StruChkFlag] = ...
    stereo_transformed: ClassVar[StruChkFlag] = ...
    template_transformed: ClassVar[StruChkFlag] = ...
    transformed: ClassVar[StruChkFlag] = ...
    values: ClassVar[dict] = ...
    __slots__: ClassVar[tuple] = ...

class StruChkResult:
    bad_set: ClassVar[StruChkResult] = ...
    names: ClassVar[dict] = ...
    success: ClassVar[StruChkResult] = ...
    transformed_set: ClassVar[StruChkResult] = ...
    values: ClassVar[dict] = ...
    __slots__: ClassVar[tuple] = ...

def CheckMolecule(RDKit) -> Any: ...
def CheckMoleculeString(*args, **kwargs) -> Any: ...
def CloseCheckMolFiles() -> Any: ...
def Generate2DCoords(RDKit) -> Any: ...
def GetAvalonCountFP(RDKit) -> Any: ...
def GetAvalonFP(RDKit) -> Any: ...
def GetAvalonFPAsWords(RDKit) -> Any: ...
def GetCanonSmiles(RDKit) -> Any: ...
def GetCheckMolLog() -> Any: ...
def InitializeCheckMol(*args, **kwargs) -> Any: ...



def log2(x): ...
def BurgesRiskBound(VCDim, nData, nWrong, conf): ...
def CristianiRiskBound(VCDim, nData, nWrong, conf): ...
def CherkasskyRiskBound(
    VCDim, nData, nWrong, conf, a1: float = ..., a2: float = ...
): ...



from _typeshed import Incomplete
from rdkit.DataStructs.VectCollection import VectCollection as VectCollection
from rdkit.ML import InfoTheory as InfoTheory
from rdkit.ML.DecTree import SigTree as SigTree

def _GenerateRandomEnsemble(nToInclude, nBits): ...
def BuildSigTree(
    examples,
    nPossibleRes,
    ensemble: Incomplete | None = ...,
    random: int = ...,
    metric=...,
    biasList=...,
    depth: int = ...,
    maxDepth: int = ...,
    useCMIM: int = ...,
    allowCollections: bool = ...,
    verbose: int = ...,
    **kwargs
): ...
def SigTreeBuilder(
    examples,
    attrs,
    nPossibleVals,
    initialVar: Incomplete | None = ...,
    ensemble: Incomplete | None = ...,
    randomDescriptors: int = ...,
    **kwargs
): ...



from _typeshed import Incomplete
from rdkit import Chem as Chem
from rdkit.Chem import rdMolDescriptors as rdMolDescriptors

HDonorSmarts: Incomplete
HAcceptorSmarts: Incomplete
HeteroatomSmarts: Incomplete
RotatableBondSmarts: Incomplete
NHOHSmarts: Incomplete
NOCountSmarts: Incomplete
NumHDonors: Incomplete
_HDonors: Incomplete
NumHAcceptors: Incomplete
_HAcceptors: Incomplete
NumHeteroatoms: Incomplete
_Heteroatoms: Incomplete
NumRotatableBonds: Incomplete
_RotatableBonds: Incomplete
NOCount: Incomplete
NHOHCount: Incomplete
RingCount: Incomplete

def HeavyAtomCount(mol): ...

_bulkConvert: Incomplete
_cfn: Incomplete
_fn: Incomplete
nm: Incomplete



from rdkit.ML.DecTree import ID3 as ID3

def GenRandomExamples(
    nVars: int = ...,
    randScale: float = ...,
    bitProb: float = ...,
    nExamples: int = ...,
    seed=...,
    addResults: int = ...,
): ...



from _typeshed import Incomplete
from collections.abc import Generator
from rdkit import Chem as Chem
from rdkit.Chem.rdDistGeom import EmbedMolecule as EmbedMolecule

class StereoEnumerationOptions:
    __slots__: Incomplete
    tryEmbedding: Incomplete
    onlyUnassigned: Incomplete
    onlyStereoGroups: Incomplete
    maxIsomers: Incomplete
    rand: Incomplete
    unique: Incomplete
    def __init__(
        self,
        tryEmbedding: bool = ...,
        onlyUnassigned: bool = ...,
        maxIsomers: int = ...,
        rand: Incomplete | None = ...,
        unique: bool = ...,
        onlyStereoGroups: bool = ...,
    ) -> None: ...

class _BondFlipper:
    bond: Incomplete
    def __init__(self, bond) -> None: ...
    def flip(self, flag) -> None: ...

class _AtomFlipper:
    atom: Incomplete
    def __init__(self, atom) -> None: ...
    def flip(self, flag) -> None: ...

class _StereoGroupFlipper:
    _original_parities: Incomplete
    def __init__(self, group) -> None: ...
    def flip(self, flag) -> None: ...

def _getFlippers(mol, options): ...

class _RangeBitsGenerator:
    nCenters: Incomplete
    def __init__(self, nCenters) -> None: ...
    def __iter__(self): ...

class _UniqueRandomBitsGenerator:
    nCenters: Incomplete
    maxIsomers: Incomplete
    rand: Incomplete
    already_seen: Incomplete
    def __init__(self, nCenters, maxIsomers, rand) -> None: ...
    def __iter__(self): ...

def GetStereoisomerCount(m, options=...): ...
def EnumerateStereoisomers(
    m, options=..., verbose: bool = ...
) -> Generator[Incomplete, None, None]: ...



LINEEND: str

def cacheImageFile(filename) -> None: ...
def preProcessImages(spec) -> None: ...
def cachedImageExists(filename): ...
def _escape(s): ...
def _normalizeLineEnds(text, desired=...): ...
def _AsciiHexEncode(input): ...
def _AsciiHexDecode(input): ...
def _AsciiHexTest(text: str = ...) -> None: ...
def _AsciiBase85Encode(input): ...
def _AsciiBase85Decode(input): ...
def _wrap(input, columns: int = ...): ...
def _AsciiBase85Test(text: str = ...) -> None: ...



from rdkit import DataStructs as DataStructs, RDConfig as RDConfig
from rdkit.Dbase.DbConnection import DbConnect as DbConnect
from rdkit.ML import CompositeRun as CompositeRun
from rdkit.ML.Data import DataUtils as DataUtils, SplitData as SplitData, Stats as Stats

__VERSION_STRING: str

def cmp(t1, t2): ...
def message(msg, noRet: int = ..., dest=...) -> None: ...
def error(msg, dest=...) -> None: ...
def ScreenModel(
    mdl, descs, data, picking=..., indices=..., errorEstimate: int = ...
): ...
def AccumulateCounts(predictions, thresh: int = ..., sortIt: int = ...): ...
def MakePlot(
    details, final, counts, pickVects, nModels, nTrueActs: int = ...
) -> None: ...
def Usage() -> None: ...



from typing import Any

def ComputeConfBox(RDKit) -> Any: ...
def ComputeConfDimsAndOffset(RDKit) -> Any: ...
def ComputeUnionBox(*args, **kwargs) -> Any: ...
def EncodeShape(RDKit, RDGeom) -> Any: ...
def ShapeProtrudeDist(*args, **kwargs) -> Any: ...
def ShapeTanimotoDist(*args, **kwargs) -> Any: ...
def ShapeTverskyIndex(*args, **kwargs) -> Any: ...



from _typeshed import Incomplete
from rdkit import Chem as Chem, DataStructs as DataStructs
from rdkit.Chem import MACCSkeys as MACCSkeys
from rdkit.ML.Cluster import Murtagh as Murtagh

def error(msg) -> None: ...
def message(msg) -> None: ...
def GetRDKFingerprint(mol): ...
def FoldFingerprintToTargetDensity(fp, **fpArgs): ...
def FingerprintMol(mol, fingerprinter=..., **fpArgs): ...
def FingerprintsFromSmiles(
    dataSource,
    idCol,
    smiCol,
    fingerprinter=...,
    reportFreq: int = ...,
    maxMols: int = ...,
    **fpArgs
): ...
def FingerprintsFromMols(
    mols, fingerprinter=..., reportFreq: int = ..., maxMols: int = ..., **fpArgs
): ...
def FingerprintsFromPickles(
    dataSource,
    idCol,
    pklCol,
    fingerprinter=...,
    reportFreq: int = ...,
    maxMols: int = ...,
    **fpArgs
): ...
def FingerprintsFromDetails(details, reportFreq: int = ...): ...

class FingerprinterDetails:
    def __init__(self) -> None: ...
    fingerprinter: Incomplete
    fpColName: str
    idName: str
    dbName: str
    outDbName: str
    tableName: str
    minSize: int
    fpSize: int
    tgtDensity: float
    minPath: int
    maxPath: int
    discrimHash: int
    useHs: int
    useValence: int
    bitsPerHash: int
    smilesName: str
    maxMols: int
    outFileName: str
    outTableName: str
    inFileName: str
    replaceTable: bool
    molPklName: str
    useSmiles: bool
    useSD: bool
    def _fingerprinterInit(self) -> None: ...
    metric: Incomplete
    doScreen: str
    topN: int
    screenThresh: float
    doThreshold: int
    smilesTableName: str
    probeSmiles: str
    probeMol: Incomplete
    noPickle: int
    def _screenerInit(self) -> None: ...
    clusterAlgo: Incomplete
    actTableName: str
    actName: str
    def _clusterInit(self) -> None: ...
    def GetMetricName(self): ...
    def SetMetricFromName(self, name) -> None: ...

def Usage() -> None: ...

_usageDoc: str

def ParseArgs(details: Incomplete | None = ...): ...



from _typeshed import Incomplete
from rdkit import Chem as Chem
from rdkit.Chem import inchi as inchi

def _is_achiral_by_symmetry(INCHI): ...

console: Incomplete
UPD_APP: Incomplete
version_re: Incomplete
reconnected_re: Incomplete
fixed_h_re: Incomplete
isotope_re: Incomplete
stereo_re: Incomplete
stereo_all_re: Incomplete
undef_stereo_re: Incomplete
all_stereo_re: Incomplete
defined_stereo_re: Incomplete
h_layer_re: Incomplete
mobile_h_group_re: Incomplete
mobile_h_atoms_re: Incomplete

class InchiInfo:
    parsed_inchi: Incomplete
    def __init__(self, inchi_str) -> None: ...
    def get_sp3_stereo(self): ...
    def get_mobile_h(self): ...



from rdkit.sping.pid import *
from _typeshed import Incomplete

f: Incomplete
_widthmaps: Incomplete
_ascents: Incomplete
_descents: Incomplete

def _closestSize(size): ...
def _pilFontPath(face, size, bold: int = ...): ...
def _matchingFontPath(font): ...
def _pilFont(font): ...

class PILCanvas(Canvas):
    _image: Incomplete
    _pen: Incomplete
    def __init__(self, size=..., name: str = ...) -> None: ...
    def __setattr__(self, attribute, value) -> None: ...
    _color: Incomplete
    def _setColor(self, c) -> None: ...
    _font: Incomplete
    def _setFont(self, font) -> None: ...
    def getImage(self): ...
    def save(
        self, file: Incomplete | None = ..., format: Incomplete | None = ...
    ) -> None: ...
    def clear(self) -> None: ...
    def stringWidth(self, s, font: Incomplete | None = ...): ...
    def fontAscent(self, font: Incomplete | None = ...): ...
    def fontDescent(self, font: Incomplete | None = ...): ...
    def drawLine(
        self,
        x1,
        y1,
        x2,
        y2,
        color: Incomplete | None = ...,
        width: Incomplete | None = ...,
        dash: Incomplete | None = ...,
        **kwargs
    ) -> None: ...
    def drawPolygon(
        self,
        pointlist,
        edgeColor: Incomplete | None = ...,
        edgeWidth: Incomplete | None = ...,
        fillColor: Incomplete | None = ...,
        closed: int = ...,
        dash: Incomplete | None = ...,
        **kwargs
    ) -> None: ...
    def drawString(
        self,
        s,
        x,
        y,
        font: Incomplete | None = ...,
        color: Incomplete | None = ...,
        angle: int = ...,
        **kwargs
    ): ...
    def drawImage(
        self,
        image,
        x1,
        y1,
        x2: Incomplete | None = ...,
        y2: Incomplete | None = ...,
        **kwargs
    ) -> None: ...

def test(): ...
def testit(canvas, s, x, y, font: Incomplete | None = ...) -> None: ...
def test2() -> None: ...



from _typeshed import Incomplete
from rdkit import Chem as Chem, RDLogger as RDLogger
from rdkit.Chem import ChemicalFeatures as ChemicalFeatures

logger: Incomplete
splitExpr: Incomplete

def GetAtomFeatInfo(factory, mol): ...
def initParser(): ...

_splashMessage: str

def existingFile(filename): ...
def processArgs(args, parser) -> None: ...
def main() -> None: ...



from _typeshed import Incomplete

class VLibNode:
    _children: Incomplete
    _parents: Incomplete
    def __init__(self, *args, **kwargs) -> None: ...
    def __iter__(self): ...
    def next(self) -> None: ...
    def reset(self) -> None: ...
    def AddChild(self, child, notify: int = ...) -> None: ...
    def RemoveChild(self, child, notify: int = ...) -> None: ...
    def GetChildren(self): ...
    def AddParent(self, parent, notify: bool = ...) -> None: ...
    def RemoveParent(self, parent, notify: bool = ...) -> None: ...
    def GetParents(self): ...
    _destroyed: bool
    def Destroy(
        self, notify: bool = ..., propagateDown: bool = ..., propagateUp: bool = ...
    ) -> None: ...

def _runDoctests(verbose: Incomplete | None = ...) -> None: ...



from _typeshed import Incomplete
from rdkit import RDConfig as RDConfig
from rdkit.Dbase import DbModule as DbModule

sqlTextTypes: Incomplete
sqlIntTypes: Incomplete
sqlFloatTypes: Incomplete
sqlBinTypes: Incomplete

def GetDbNames(
    user: str = ...,
    password: str = ...,
    dirName: str = ...,
    dBase: str = ...,
    cn: Incomplete | None = ...,
): ...
def GetTableNames(
    dBase,
    user: str = ...,
    password: str = ...,
    includeViews: int = ...,
    cn: Incomplete | None = ...,
): ...
def GetColumnInfoFromCursor(cursor): ...
def GetColumnNamesAndTypes(
    dBase,
    table,
    user: str = ...,
    password: str = ...,
    join: str = ...,
    what: str = ...,
    cn: Incomplete | None = ...,
): ...
def GetColumnNames(
    dBase,
    table,
    user: str = ...,
    password: str = ...,
    join: str = ...,
    what: str = ...,
    cn: Incomplete | None = ...,
): ...



from _typeshed import Incomplete

class VisOpts:
    circRad: int
    minCircRad: int
    maxCircRad: int
    circColor: Incomplete
    terminalEmptyColor: Incomplete
    terminalOnColor: Incomplete
    terminalOffColor: Incomplete
    outlineColor: Incomplete
    lineColor: Incomplete
    lineWidth: int
    horizOffset: int
    vertOffset: int
    labelFont: Incomplete
    highlightColor: Incomplete
    highlightWidth: int

visOpts: Incomplete

def CalcTreeNodeSizes(node) -> None: ...
def _ExampleCounter(node, min, max): ...
def _ApplyNodeScales(node, min, max) -> None: ...
def SetNodeScales(node) -> None: ...
def DrawTreeNode(
    node, loc, canvas, nRes: int = ..., scaleLeaves: bool = ..., showPurity: bool = ...
) -> None: ...
def CalcTreeWidth(tree): ...
def DrawTree(
    tree,
    canvas,
    nRes: int = ...,
    scaleLeaves: bool = ...,
    allowShrink: bool = ...,
    showPurity: bool = ...,
) -> None: ...
def ResetTree(tree) -> None: ...
def _simpleTest(canv) -> None: ...



from _typeshed import Incomplete
from rdkit.ML.Data import DataUtils as DataUtils

class Composite:
    modelList: Incomplete
    errList: Incomplete
    countList: Incomplete
    modelVotes: Incomplete
    quantBounds: Incomplete
    nPossibleVals: Incomplete
    quantizationRequirements: Incomplete
    _descNames: Incomplete
    _mapOrder: Incomplete
    activityQuant: Incomplete
    def __init__(self) -> None: ...
    _modelFilterFrac: Incomplete
    _modelFilterVal: Incomplete
    def SetModelFilterData(
        self, modelFilterFrac: float = ..., modelFilterVal: float = ...
    ) -> None: ...
    def SetDescriptorNames(self, names) -> None: ...
    def GetDescriptorNames(self): ...
    def SetQuantBounds(self, qBounds, nPossible: Incomplete | None = ...) -> None: ...
    def GetQuantBounds(self): ...
    def GetActivityQuantBounds(self): ...
    def SetActivityQuantBounds(self, bounds) -> None: ...
    def QuantizeActivity(
        self, example, activityQuant: Incomplete | None = ..., actCol: int = ...
    ): ...
    def QuantizeExample(self, example, quantBounds: Incomplete | None = ...): ...
    def MakeHistogram(self): ...
    def CollectVotes(
        self,
        example,
        quantExample,
        appendExample: int = ...,
        onlyModels: Incomplete | None = ...,
    ): ...
    def ClassifyExample(
        self,
        example,
        threshold: int = ...,
        appendExample: int = ...,
        onlyModels: Incomplete | None = ...,
    ): ...
    def GetVoteDetails(self): ...
    def _RemapInput(self, inputVect): ...
    def GetInputOrder(self): ...
    def SetInputOrder(self, colNames) -> None: ...
    def Grow(
        self,
        examples,
        attrs,
        nPossibleVals,
        buildDriver,
        pruner: Incomplete | None = ...,
        nTries: int = ...,
        pruneIt: int = ...,
        needsQuantization: int = ...,
        progressCallback: Incomplete | None = ...,
        **buildArgs
    ) -> None: ...
    def ClearModelExamples(self) -> None: ...
    def Pickle(self, fileName: str = ..., saveExamples: int = ...) -> None: ...
    def AddModel(self, model, error, needsQuantization: int = ...) -> None: ...
    def AverageErrors(self): ...
    def SortModels(self, sortOnError: bool = ...) -> None: ...
    def GetModel(self, i): ...
    def SetModel(self, i, val) -> None: ...
    def GetCount(self, i): ...
    def SetCount(self, i, val) -> None: ...
    def GetError(self, i): ...
    def SetError(self, i, val) -> None: ...
    def GetDataTuple(self, i): ...
    def SetDataTuple(self, i, tup) -> None: ...
    def GetAllData(self): ...
    def __len__(self) -> int: ...
    def __getitem__(self, which): ...
    def __str__(self): ...



from typing import Any, ClassVar

class MolCatalog:
    __instance_size__: ClassVar[int] = ...
    __safe_for_unpickling__: ClassVar[bool] = ...
    @classmethod
    def __init__(cls, *args, **kwargs) -> None: ...
    @classmethod
    def AddEdge(cls, *args, **kwargs) -> Any: ...
    @classmethod
    def AddEntry(cls, *args, **kwargs) -> Any: ...
    @classmethod
    def GetBitDescription(cls, *args, **kwargs) -> Any: ...
    @classmethod
    def GetBitEntryId(cls, *args, **kwargs) -> Any: ...
    @classmethod
    def GetEntryBitId(cls, *args, **kwargs) -> Any: ...
    @classmethod
    def GetEntryDescription(cls, *args, **kwargs) -> Any: ...
    @classmethod
    def GetEntryDownIds(cls, *args, **kwargs) -> Any: ...
    @classmethod
    def GetFPLength(cls, *args, **kwargs) -> Any: ...
    @classmethod
    def GetNumEntries(cls, *args, **kwargs) -> Any: ...
    @classmethod
    def Serialize(cls, *args, **kwargs) -> Any: ...
    @classmethod
    def __getinitargs__(cls) -> Any: ...
    @classmethod
    def __reduce__(cls) -> Any: ...

class MolCatalogEntry:
    __instance_size__: ClassVar[int] = ...
    __safe_for_unpickling__: ClassVar[bool] = ...
    @classmethod
    def __init__(cls, *args, **kwargs) -> None: ...
    @classmethod
    def GetDescription(cls, RDKit) -> Any: ...
    @classmethod
    def GetMol(cls, RDKit) -> Any: ...
    @classmethod
    def GetOrder(cls, RDKit) -> Any: ...
    @classmethod
    def SetDescription(cls, *args, **kwargs) -> Any: ...
    @classmethod
    def SetMol(cls, *args, **kwargs) -> Any: ...
    @classmethod
    def SetOrder(cls, RDKit, unsignedint) -> Any: ...
    @classmethod
    def __getinitargs__(cls, RDKit) -> Any: ...
    @classmethod
    def __reduce__(cls) -> Any: ...

def CreateMolCatalog() -> Any: ...



from types import TracebackType
from typing import (
    IO,
    Any,
    ClassVar,
    Iterable,
    List,
    Optional,
    TextIO,
    Tuple,
    Union,
    overload,
)

from rdkit.Chem import Mol
from rdkit.Chem.rdMolDescriptors import AtomPairsParameters

class CXSmilesFields:
    CX_ALL: ClassVar[CXSmilesFields] = ...
    CX_ATOM_LABELS: ClassVar[CXSmilesFields] = ...
    CX_ATOM_PROPS: ClassVar[CXSmilesFields] = ...
    CX_COORDS: ClassVar[CXSmilesFields] = ...
    CX_ENHANCEDSTEREO: ClassVar[CXSmilesFields] = ...
    CX_LINKNODES: ClassVar[CXSmilesFields] = ...
    CX_MOLFILE_VALUES: ClassVar[CXSmilesFields] = ...
    CX_NONE: ClassVar[CXSmilesFields] = ...
    CX_POLYMER: ClassVar[CXSmilesFields] = ...
    CX_RADICALS: ClassVar[CXSmilesFields] = ...
    CX_SGROUPS: ClassVar[CXSmilesFields] = ...
    names: ClassVar[dict] = ...
    values: ClassVar[dict] = ...
    __slots__: ClassVar[tuple] = ...

class ForwardSDMolSupplier:
    @overload
    def __init__(
        self,
        fileobj: IO,
        sanitize: bool = ...,
        removeHs: bool = ...,
        strictParsing: bool = ...,
    ): ...
    @overload
    def __init__(
        self,
        filename: str,
        sanitize: bool = ...,
        removeHs: bool = ...,
        strictParsing: bool = ...,
    ): ...
    def GetEOFHitOnRead(self) -> bool: ...
    def GetProcessPropertyLists(self) -> bool: ...
    def SetProcessPropertyLists(self, arg2: bool): ...
    def atEnd(self) -> bool: ...
    def __enter__(self) -> "ForwardSDMolSupplier": ...
    def __exit__(
        self,
        type: type[BaseException],
        value: BaseException,
        traceback: TracebackType,
    ) -> Optional[bool]: ...
    def __iter__(self) -> Any: ...
    def __next__(self) -> Optional[Mol]: ...
    def __reduce__(self) -> Any: ...

class MaeMolSupplier:
    @overload
    def __init__(self, fileobj: IO, sanitize: bool = ..., removeHs: bool = ...): ...
    @overload
    def __init__(self, filename: str, sanitize: bool = ..., removeHs: bool = ...): ...
    def atEnd(self) -> bool: ...
    def __enter__(self) -> "MaeMolSupplier": ...
    def __exit__(
        self,
        type: type[BaseException],
        value: BaseException,
        traceback: TracebackType,
    ) -> Optional[bool]: ...
    def __iter__(self) -> Any: ...
    def __next__(self) -> Optional[Mol]: ...
    def __reduce__(self) -> Any: ...

class MultithreadedSDMolSupplier:
    __instance_size__: ClassVar[int] = ...
    def __init__(
        self,
        fileName: str,
        sanitize: bool = ...,
        removeHs: bool = ...,
        strictParsing: bool = True,
        numWriterThreads: int = ...,
        sizeInputQueue: int = ...,
        sizeOutputQueue: int = ...,
    ): ...
    def GetLastItemText(self) -> str: ...
    def GetLastRecordId(self) -> int: ...
    def GetProcessPropertyLists(self) -> bool: ...
    def SetProcessPropertyLists(self, arg2: bool): ...
    def atEnd(self) -> bool: ...
    def __enter__(self) -> "MultithreadedSDMolSupplier": ...
    def __exit__(
        self,
        type: type[BaseException],
        value: BaseException,
        traceback: TracebackType,
    ) -> Optional[bool]: ...
    def __iter__(self) -> Any: ...
    def __next__(self) -> Optional[Mol]: ...
    def __reduce__(self) -> Any: ...

class MultithreadedSmilesMolSupplier:
    __instance_size__: ClassVar[int] = ...
    def __init__(
        self,
        fileName: str,
        delimiter: str = ...,
        smilesColumn: int = ...,
        nameColumn: int = ...,
        titleLine: bool = ...,
        sanitize: bool = ...,
        numWriterThreads: int = ...,
        sizeInputQueue: int = ...,
        sizeOutputQueue: int = ...,
    ): ...
    def GetLastItemText(self) -> str: ...
    def GetLastRecordId(self) -> int: ...
    def atEnd(self) -> bool: ...
    def __enter__(self) -> "MultithreadedSmilesMolSupplier": ...
    def __exit__(
        self,
        type: type[BaseException],
        value: BaseException,
        traceback: TracebackType,
    ) -> Optional[bool]: ...
    def __iter__(self) -> Any: ...
    def __next__(self) -> Optional[Mol]: ...
    def __reduce__(self) -> Any: ...

class PDBWriter:
    def __init__(self, fileName: str, flavor: int = ...): ...
    def NumMols(self) -> int: ...
    def close(self): ...
    def flush(self): ...
    def write(self, mol: Mol, confId: int = ...): ...
    def __enter__(self) -> "PDBWriter": ...
    def __exit__(
        self,
        type: type[BaseException],
        value: BaseException,
        traceback: TracebackType,
    ) -> Optional[bool]: ...
    def __reduce__(self) -> Any: ...

class SDMolSupplier:
    __instance_size__: ClassVar[int] = ...
    def __init__(
        self,
        fileName: str,
        sanitize: bool = ...,
        removeHs: bool = ...,
        strictParsing: bool = ...,
    ): ...
    def GetItemText(self, index: int) -> str: ...
    def GetProcessPropertyLists(self) -> bool: ...
    def SetData(
        self,
        data: str,
        sanitize: bool = ...,
        removeHs: bool = ...,
        strictParsing: bool = ...,
    ): ...
    def SetProcessPropertyLists(self, arg1: bool): ...
    def atEnd(self) -> bool: ...
    def reset(self) -> Any: ...
    def __enter__(self) -> "SDMolSupplier": ...
    def __exit__(
        self,
        type: type[BaseException],
        value: BaseException,
        traceback: TracebackType,
    ) -> Optional[bool]: ...
    def __iter__(self) -> Any: ...
    def __next__(self) -> Optional[Mol]: ...
    def __reduce__(self) -> Any: ...
    def __len__(self) -> int: ...
    def __getitem__(self, int) -> Mol: ...

class SDWriter:
    @overload
    def __init__(self, fileName: str): ...
    @overload
    def __init__(self, _: TextIO): ...
    def GetForceV3000(self) -> bool: ...
    def GetKekulize(self) -> bool: ...
    @staticmethod
    def GetText(
        mol: Mol,
        confId: int = ...,
        kekulize: bool = ...,
        force_v3000: bool = ...,
        molid: int = ...,
    ) -> str: ...
    def NumMols(self) -> int: ...
    def SetForceV3000(self, arg2: bool): ...
    def SetKekulize(self, arg2: bool): ...
    def SetProps(
        self,
        arg2: Union[List[str], Tuple[str, ...]],
    ) -> Any: ...
    def close(self) -> None: ...
    def flush(self) -> None: ...
    def write(self, mol: Mol, confId: int = ...) -> None: ...
    def __enter__(self) -> "SDWriter": ...
    def __exit__(
        self,
        type: type[BaseException],
        value: BaseException,
        traceback: TracebackType,
    ) -> Optional[bool]: ...
    def __reduce__(self) -> Any: ...

class SmartsParserParams:
    __instance_size__: ClassVar[int] = ...
    allowCXSMILES: Any
    debugParse: Any
    mergeHs: Any
    parseName: Any
    strictCXSMILES: Any
    @classmethod
    def __init__(cls, *args, **kwargs) -> None: ...
    @classmethod
    def __reduce__(cls) -> Any: ...

class SmilesMolSupplier:
    __instance_size__: ClassVar[int] = ...
    @classmethod
    def __init__(cls, *args, **kwargs) -> None: ...
    @classmethod
    def GetItemText(cls, RDKit, unsignedint) -> Any: ...
    @classmethod
    def SetData(cls, *args, **kwargs) -> Any: ...
    @classmethod
    def reset(cls, RDKit) -> Any: ...
    @classmethod
    def __enter__(cls, RDKit) -> Any: ...
    @classmethod
    def __exit__(cls, type, value, traceback) -> Any: ...
    @classmethod
    def __getitem__(cls, RDKit, int) -> Any: ...
    @classmethod
    def __iter__(cls, RDKit) -> Any: ...
    @classmethod
    def __len__(cls, RDKit) -> Any: ...
    @classmethod
    def __next__(cls, RDKit) -> Any: ...
    @classmethod
    def __reduce__(cls) -> Any: ...

class SmilesParserParams:
    __instance_size__: ClassVar[int] = ...
    allowCXSMILES: Any
    debugParse: Any
    parseName: Any
    removeHs: Any
    sanitize: Any
    strictCXSMILES: Any
    useLegacyStereo: Any
    @classmethod
    def __init__(cls, *args, **kwargs) -> None: ...
    @classmethod
    def __reduce__(cls) -> Any: ...

class SmilesWriteParams:
    __instance_size__: ClassVar[int] = ...
    allBondsExplicit: Any
    allHsExplicit: Any
    canonical: Any
    doIsomericSmiles: Any
    doKekule: Any
    doRandom: Any
    rootedAtAtom: Any
    @classmethod
    def __init__(cls, *args, **kwargs) -> None: ...
    @classmethod
    def __reduce__(cls) -> Any: ...

class SmilesWriter:
    @classmethod
    def __init__(cls, *args, **kwargs) -> None: ...
    @classmethod
    def NumMols(cls, RDKit) -> Any: ...
    @classmethod
    def SetProps(cls, RDKit, boost) -> Any: ...
    @classmethod
    def close(cls, RDKit) -> Any: ...
    @classmethod
    def flush(cls, RDKit) -> Any: ...
    @classmethod
    def write(cls, *args, **kwargs) -> Any: ...
    @classmethod
    def __enter__(cls, RDKit) -> Any: ...
    @classmethod
    def __exit__(cls, type, value, traceback) -> Any: ...
    @classmethod
    def __reduce__(cls) -> Any: ...

class TDTMolSupplier:
    __instance_size__: ClassVar[int] = ...
    @classmethod
    def __init__(cls, *args, **kwargs) -> None: ...
    @classmethod
    def GetItemText(cls, RDKit, unsignedint) -> Any: ...
    @classmethod
    def SetData(cls, *args, **kwargs) -> Any: ...
    @classmethod
    def reset(cls, RDKit) -> Any: ...
    @classmethod
    def __enter__(cls, RDKit) -> Any: ...
    @classmethod
    def __exit__(cls, type, value, traceback) -> Any: ...
    @classmethod
    def __getitem__(cls, RDKit, int) -> Any: ...
    @classmethod
    def __iter__(cls, RDKit) -> Any: ...
    @classmethod
    def __len__(cls, RDKit) -> Any: ...
    @classmethod
    def __next__(cls, RDKit) -> Any: ...
    @classmethod
    def __reduce__(cls) -> Any: ...

class TDTWriter:
    @classmethod
    def __init__(cls, *args, **kwargs) -> None: ...
    @classmethod
    def GetNumDigits(cls, RDKit) -> Any: ...
    @classmethod
    def GetWrite2D(cls, RDKit) -> Any: ...
    @classmethod
    def GetWriteNames(cls, RDKit) -> Any: ...
    @classmethod
    def NumMols(cls, RDKit) -> Any: ...
    @classmethod
    def SetNumDigits(cls, RDKit, unsignedint) -> Any: ...
    @classmethod
    def SetProps(cls, RDKit, boost) -> Any: ...
    @classmethod
    def SetWrite2D(cls, RDKit) -> Any: ...
    @classmethod
    def SetWriteNames(cls, RDKit) -> Any: ...
    @classmethod
    def close(cls, RDKit) -> Any: ...
    @classmethod
    def flush(cls, RDKit) -> Any: ...
    @classmethod
    def write(cls, *args, **kwargs) -> Any: ...
    @classmethod
    def __enter__(cls, RDKit) -> Any: ...
    @classmethod
    def __exit__(cls, type, value, traceback) -> Any: ...
    @classmethod
    def __reduce__(cls) -> Any: ...

def AddMetadataToPNGFile(*args, **kwargs) -> Any: ...
def AddMetadataToPNGString(*args, **kwargs) -> Any: ...
def AtomFromSmarts(*args, **kwargs) -> Any: ...
def AtomFromSmiles(*args, **kwargs) -> Any: ...
def BondFromSmarts(*args, **kwargs) -> Any: ...
def BondFromSmiles(*args, **kwargs) -> Any: ...
@overload
def CanonicalRankAtoms(mol, breakTies=...) -> Any: ...
@overload
def CanonicalRankAtoms(RDKit) -> Any: ...
@overload
def CanonicalRankAtomsInFragment(mol, atomsToUse=..., breakTies=...) -> Any: ...
@overload
def CanonicalRankAtomsInFragment(RDKit, boost) -> Any: ...
def CreateAtomBoolPropertyList(*args, **kwargs) -> Any: ...
def CreateAtomDoublePropertyList(*args, **kwargs) -> Any: ...
def CreateAtomIntPropertyList(*args, **kwargs) -> Any: ...
def CreateAtomStringPropertyList(*args, **kwargs) -> Any: ...
def MetadataFromPNGFile(boost) -> Any: ...
def MetadataFromPNGString(boost) -> Any: ...
def MolFragmentToCXSmarts(RDKit, boost) -> Any: ...
def MolFragmentToCXSmiles(RDKit, boost) -> Any: ...
def MolFragmentToSmarts(RDKit, boost) -> Any: ...
def MolFragmentToSmiles(
    mol: Mol,
    atomsToUse: Iterable[int],
    bondsToUse: Optional[Iterable[int]] = None,
    atomSymbols: Optional[Iterable[str]] = None,
    bondSymbols: Optional[Iterable[str]] = None,
    isomericSmiles: Optional[bool] = True,
    kekuleSmiles: Optional[bool] = False,
    rootedAtAtom: Optional[int] = -1,
    canonical: Optional[bool] = True,
    allBondsExplicit: Optional[bool] = False,
    allHsExplicit: Optional[bool] = False,
    doRandom: Optional[bool] = False,
) -> str: ...
def MolFromFASTA(*args, **kwargs) -> Any: ...
def MolFromHELM(boost) -> Any: ...
def MolFromMol2Block(*args, **kwargs) -> Any: ...
def MolFromMol2File(*args, **kwargs) -> Any: ...
def MolFromMolBlock(boost) -> Any: ...
def MolFromMolFile(*args, **kwargs) -> Any: ...
def MolFromPDBBlock(boost) -> Any: ...
def MolFromPDBFile(*args, **kwargs) -> Any: ...
def MolFromPNGFile(*args, **kwargs) -> Any: ...
def MolFromPNGString(boost) -> Any: ...
def MolFromRDKitSVG(boost) -> Any: ...
def MolFromSequence(boost) -> Any: ...
@overload
def MolFromSmarts(boost) -> Any: ...
@overload
def MolFromSmarts(boost, RDKit) -> Any: ...
def MolFromSmiles(
    SMILES: str, sanitize: bool = ..., replacements: dict[str, str] = ...
) -> Mol | None: ...
def MolFromTPLBlock(boost) -> Any: ...
def MolFromTPLFile(*args, **kwargs) -> Any: ...
def MolMetadataToPNGFile(RDKit, boost) -> Any: ...
def MolMetadataToPNGString(RDKit, boost) -> Any: ...
def MolToCMLBlock(RDKit) -> Any: ...
def MolToCMLFile(*args, **kwargs) -> Any: ...
def MolToCXSmarts(RDKit) -> Any: ...
def MolToCXSmiles(RDKit) -> Any: ...
def MolToFASTA(RDKit) -> Any: ...
def MolToHELM(RDKit) -> Any: ...
def MolToMolBlock(
    mol: Mol,
    includeStereo: bool = ...,
    confId: int = ...,
    kekulize: bool = ...,
    forceV3000: bool = ...,
) -> str: ...
def MolToMolFile(*args, **kwargs) -> Any: ...
def MolToPDBBlock(RDKit) -> Any: ...
def MolToPDBFile(*args, **kwargs) -> Any: ...
def MolToRandomSmilesVect(*args, **kwargs) -> Any: ...
def MolToSequence(RDKit) -> Any: ...
def MolToSmarts(RDKit) -> Any: ...
def MolToSmiles(
    mol: Mol,
    isomericSmiles: Optional[bool] = True,
    kekuleSmiles: Optional[bool] = False,
    rootedAtAtom: Optional[int] = -1,
    canonical: Optional[bool] = True,
    allBondsExplicit: Optional[bool] = False,
    allHsExplicit: Optional[bool] = False,
    doRandom: Optional[bool] = False,
) -> str: ...
def MolToTPLBlock(RDKit) -> Any: ...
def MolToTPLFile(*args, **kwargs) -> Any: ...
def MolToV3KMolBlock(RDKit) -> Any: ...
def MolToV3KMolFile(*args, **kwargs) -> Any: ...
def MolToXYZBlock(RDKit) -> Any: ...
def MolToXYZFile(*args, **kwargs) -> Any: ...
def MolsFromPNGFile(*args, **kwargs) -> Any: ...
def MolsFromPNGString(boost) -> Any: ...
def SmilesMolSupplierFromText(*args, **kwargs) -> Any: ...



from _typeshed import Incomplete
from rdkit.Chem.Pharm2D import Utils as Utils
from rdkit.DataStructs.cDataStructs import (
    IntSparseIntVect as IntSparseIntVect,
    LongSparseIntVect as LongSparseIntVect,
    SparseBitVect as SparseBitVect,
)

_verbose: bool

class SigFactory:
    featFactory: Incomplete
    useCounts: Incomplete
    minPointCount: Incomplete
    maxPointCount: Incomplete
    shortestPathsOnly: Incomplete
    includeBondOrder: Incomplete
    trianglePruneBins: Incomplete
    skipFeats: Incomplete
    _bins: Incomplete
    sigKlass: Incomplete
    def __init__(
        self,
        featFactory,
        useCounts: bool = ...,
        minPointCount: int = ...,
        maxPointCount: int = ...,
        shortestPathsOnly: bool = ...,
        includeBondOrder: bool = ...,
        skipFeats: Incomplete | None = ...,
        trianglePruneBins: bool = ...,
    ) -> None: ...
    def SetBins(self, bins) -> None: ...
    def GetBins(self): ...
    def GetNumBins(self): ...
    def GetSignature(self): ...
    def _GetBitSummaryData(self, bitIdx): ...
    def GetBitDescriptionAsText(
        self, bitIdx, includeBins: int = ..., fullPage: int = ...
    ) -> None: ...
    def GetBitDescription(self, bitIdx): ...
    def _findBinIdx(self, dists, bins, scaffolds): ...
    def GetFeatFamilies(self): ...
    def GetMolFeats(self, mol): ...
    def GetBitIdx(self, featIndices, dists, sortIndices: bool = ...): ...
    def GetBitInfo(self, idx): ...
    _scaffolds: Incomplete
    _starts: Incomplete
    _nFeats: Incomplete
    _sigSize: Incomplete
    def Init(self) -> None: ...
    def GetSigSize(self): ...



from _typeshed import Incomplete
from rdkit import Chem as Chem, RDConfig as RDConfig
from rdkit.Chem import rdMolDescriptors as rdMolDescriptors
from rdkit.Chem.rdMolDescriptors import (
    GetAtomPairFingerprint as GetAtomPairFingerprint,
    GetTopologicalTorsionFingerprint as GetTopologicalTorsionFingerprint,
)

numPathBits: Incomplete
_maxPathLen: Incomplete
numFpBits: Incomplete
fpLen: Incomplete

def _readPattyDefs(fname=...): ...

_pattyDefs: Incomplete

def AssignPattyTypes(mol, defns: Incomplete | None = ...): ...

typMap: Incomplete

def GetBPFingerprint(mol, fpfn=...): ...
def GetBTFingerprint(mol, fpfn=...): ...
def _runDoctests(verbose: Incomplete | None = ...) -> None: ...



from _typeshed import Incomplete
from rdkit.ML.Neural import ActFuncs as ActFuncs, NetNode as NetNode

class Network:
    def ConstructRandomWeights(
        self, minWeight: int = ..., maxWeight: int = ...
    ) -> None: ...
    nConnections: Incomplete
    def FullyConnectNodes(self) -> None: ...
    nodeCounts: Incomplete
    numInputNodes: Incomplete
    numOutputNodes: Incomplete
    numHiddenLayers: Incomplete
    numInHidden: Incomplete
    nodeList: Incomplete
    layerIndices: Incomplete
    def ConstructNodes(self, nodeCounts, actFunc, actFuncParms) -> None: ...
    def GetInputNodeList(self): ...
    def GetOutputNodeList(self): ...
    def GetHiddenLayerNodeList(self, which): ...
    def GetNumNodes(self): ...
    def GetNumHidden(self): ...
    def GetNode(self, which): ...
    def GetAllNodes(self): ...
    lastResults: Incomplete
    def ClassifyExample(self, example, appendExamples: int = ...): ...
    def GetLastOutputs(self): ...
    def __str__(self): ...
    def __init__(
        self,
        nodeCounts,
        nodeConnections: Incomplete | None = ...,
        actFunc=...,
        actFuncParms=...,
        weightBounds: int = ...,
    ) -> None: ...



from _typeshed import Incomplete

class Trainer: ...

class BackProp(Trainer):
    oldDeltaW: Incomplete
    def StepUpdate(self, example, net, resVect: Incomplete | None = ...): ...
    def TrainOnLine(
        self,
        examples,
        net,
        maxIts: int = ...,
        errTol: float = ...,
        useAvgErr: int = ...,
        silent: int = ...,
    ) -> None: ...
    speed: Incomplete
    momentum: Incomplete
    def __init__(self, speed: float = ..., momentum: float = ...) -> None: ...



from _typeshed import Incomplete
from rdkit.ML.KNN import KNNModel as KNNModel

class KNNRegressionModel(KNNModel.KNNModel):
    _badExamples: Incomplete
    def __init__(self, k, attrs, dfunc, radius: Incomplete | None = ...) -> None: ...
    def type(self): ...
    def SetBadExamples(self, examples) -> None: ...
    def GetBadExamples(self): ...
    def NameModel(self, varNames) -> None: ...
    def PredictExample(
        self,
        example,
        appendExamples: int = ...,
        weightedAverage: int = ...,
        neighborList: Incomplete | None = ...,
    ): ...



from _typeshed import Incomplete
from rdkit import Chem as Chem
from rdkit.Chem import (
    Crippen as Crippen,
    rdMolDescriptors as rdMolDescriptors,
    rdPartialCharges as rdPartialCharges,
)

ptable: Incomplete
bondScaleFacts: Incomplete

def _LabuteHelper(mol, includeHs: int = ..., force: int = ...): ...
def _pyLabuteHelper(mol, includeHs: int = ..., force: int = ...): ...

mrBins: Incomplete

def pySMR_VSA_(mol, bins: Incomplete | None = ..., force: int = ...): ...

SMR_VSA_: Incomplete
logpBins: Incomplete

def pySlogP_VSA_(mol, bins: Incomplete | None = ..., force: int = ...): ...

SlogP_VSA_: Incomplete
chgBins: Incomplete

def pyPEOE_VSA_(mol, bins: Incomplete | None = ..., force: int = ...): ...

PEOE_VSA_: Incomplete

def _InstallDescriptors(): ...
def pyLabuteASA(mol, includeHs: int = ...): ...

LabuteASA: Incomplete

def _pyTPSAContribs(mol, verbose: bool = ...): ...
def _pyTPSA(mol, verbose: bool = ...): ...

TPSA: Incomplete



from .utils import memoized_property as memoized_property
from _typeshed import Incomplete
from rdkit import Chem as Chem

log: Incomplete

class AcidBasePair:
    name: Incomplete
    acid_str: Incomplete
    base_str: Incomplete
    def __init__(self, name, acid, base) -> None: ...
    def acid(self): ...
    def base(self): ...
    def __repr__(self): ...
    def __str__(self): ...

ACID_BASE_PAIRS: Incomplete

class ChargeCorrection:
    name: Incomplete
    smarts_str: Incomplete
    charge: Incomplete
    def __init__(self, name, smarts, charge) -> None: ...
    def smarts(self): ...
    def __repr__(self): ...
    def __str__(self): ...

CHARGE_CORRECTIONS: Incomplete

class Reionizer:
    acid_base_pairs: Incomplete
    charge_corrections: Incomplete
    def __init__(self, acid_base_pairs=..., charge_corrections=...) -> None: ...
    def __call__(self, mol): ...
    def reionize(self, mol): ...
    def _strongest_protonated(self, mol): ...
    def _weakest_ionized(self, mol): ...

class Uncharger:
    _pos_h: Incomplete
    _pos_quat: Incomplete
    _neg: Incomplete
    _neg_acid: Incomplete
    def __init__(self) -> None: ...
    def __call__(self, mol): ...
    def uncharge(self, mol): ...



from _typeshed import Incomplete
from typing import Any, ClassVar, List, Tuple

from typing import overload
import rdkit.rdBase
from rdkit.Chem import Mol

_BCUT2D_version: str
_CalcAUTOCORR2D_version: str
_CalcAUTOCORR3D_version: str
_CalcAsphericity_version: str
_CalcChi0n_version: str
_CalcChi0v_version: str
_CalcChi1n_version: str
_CalcChi1v_version: str
_CalcChi2n_version: str
_CalcChi2v_version: str
_CalcChi3n_version: str
_CalcChi3v_version: str
_CalcChi4n_version: str
_CalcChi4v_version: str
_CalcChiNn_version: str
_CalcChiNv_version: str
_CalcCoulombMat_version: str
_CalcCrippenDescriptors_version: str
_CalcEMMcharges_version: str
_CalcEccentricity_version: str
_CalcExactMolWt_version: str
_CalcFractionCSP3_version: str
_CalcGETAWAY_version: str
_CalcHallKierAlpha_version: str
_CalcInertialShapeFactor_version: str
_CalcKappa1_version: str
_CalcKappa2_version: str
_CalcKappa3_version: str
_CalcLabuteASA_version: str
_CalcMORSE_version: str
_CalcMolFormula_version: str
_CalcMolWt_version: str
_CalcNPR1_version: str
_CalcNPR2_version: str
_CalcNumAliphaticCarbocycles_version: str
_CalcNumAliphaticHeterocycles_version: str
_CalcNumAliphaticRings_version: str
_CalcNumAmideBonds_version: str
_CalcNumAromaticCarbocycles_version: str
_CalcNumAromaticHeterocycles_version: str
_CalcNumAromaticRings_version: str
_CalcNumAtomStereoCenters_version: str
_CalcNumAtoms_version: str
_CalcNumBridgeheadAtoms_version: str
_CalcNumHBA_version: str
_CalcNumHBD_version: str
_CalcNumHeavyAtoms_version: str
_CalcNumHeteroatoms_version: str
_CalcNumHeterocycles_version: str
_CalcNumLipinskiHBA_version: str
_CalcNumLipinskiHBD_version: str
_CalcNumRings_version: str
_CalcNumRotatableBonds_version: str
_CalcNumSaturatedCarbocycles_version: str
_CalcNumSaturatedHeterocycles_version: str
_CalcNumSaturatedRings_version: str
_CalcNumSpiroAtoms_version: str
_CalcNumUnspecifiedAtomStereoCenters_version: str
_CalcPBF_version: str
_CalcPMI1_version: str
_CalcPMI2_version: str
_CalcPMI3_version: str
_CalcPhi_version: str
_CalcRDF_version: str
_CalcRadiusOfGyration_version: str
_CalcSpherocityIndex_version: str
_CalcTPSA_version: str
_CalcWHIM_version: str
_ConnectivityInvariants_version: str
_FeatureInvariants_version: str
_GetAtomFeatures_version: str
_MorganFingerprint_version: str

class AtomPairsParameters:
    atomTypes: ClassVar[rdkit.rdBase._vectj] = ...
    codeSize: ClassVar[int] = ...
    numAtomPairFingerprintBits: ClassVar[int] = ...
    numBranchBits: ClassVar[int] = ...
    numChiralBits: ClassVar[int] = ...
    numPathBits: ClassVar[int] = ...
    numPiBits: ClassVar[int] = ...
    numTypeBits: ClassVar[int] = ...
    version: ClassVar[str] = ...
    __instance_size__: ClassVar[int] = ...
    @classmethod
    def __init__(cls, *args, **kwargs) -> None: ...
    @classmethod
    def __reduce__(cls) -> Any: ...

class NumRotatableBondsOptions:
    Default: ClassVar[NumRotatableBondsOptions] = ...
    NonStrict: ClassVar[NumRotatableBondsOptions] = ...
    Strict: ClassVar[NumRotatableBondsOptions] = ...
    StrictLinkages: ClassVar[NumRotatableBondsOptions] = ...
    names: ClassVar[dict] = ...
    values: ClassVar[dict] = ...
    __slots__: ClassVar[tuple] = ...

class Properties:
    __instance_size__: ClassVar[int] = ...
    @classmethod
    def __init__(cls, *args, **kwargs) -> None: ...
    @classmethod
    def AnnotateProperties(cls, *args, **kwargs) -> Any: ...
    @classmethod
    def ComputeProperties(cls, *args, **kwargs) -> Any: ...
    def GetAvailableProperties(self, *args, **kwargs) -> Any: ...
    def GetProperty(self, *args, **kwargs) -> Any: ...
    @classmethod
    def GetPropertyNames(cls, RDKit) -> Any: ...
    def RegisterProperty(self, *args, **kwargs) -> Any: ...
    @classmethod
    def __reduce__(cls) -> Any: ...

class PropertyFunctor:
    @classmethod
    def __init__(cls, *args, **kwargs) -> None: ...
    @classmethod
    def GetName(cls, RDKit) -> Any: ...
    @classmethod
    def GetVersion(cls, RDKit) -> Any: ...
    def __call__(self, *args, **kwargs) -> Any: ...
    @classmethod
    def __reduce__(cls) -> Any: ...

class PropertyRangeQuery:
    @classmethod
    def __init__(cls, *args, **kwargs) -> None: ...
    @classmethod
    def Match(cls, *args, **kwargs) -> Any: ...
    @classmethod
    def __reduce__(cls) -> Any: ...

class PythonPropertyFunctor(PropertyFunctor):
    __instance_size__: ClassVar[int] = ...
    @classmethod
    def __init__(cls, *args, **kwargs) -> None: ...
    def __call__(self, RDKit) -> Any: ...
    @classmethod
    def __reduce__(cls) -> Any: ...

@overload
def BCUT2D(mol: Mol, atom_props: List[Incomplete]) -> Any: ...
@overload
def BCUT2D(mol: Mol, atom_props: Tuple[Incomplete, ...]) -> Any: ...
@overload
def BCUT2D(mol: Mol, atom_propname: str) -> Any: ...
def CalcAUTOCORR2D(RDKit) -> Any: ...
def CalcAUTOCORR3D(RDKit) -> Any: ...
def CalcAsphericity(RDKit) -> Any: ...
def CalcChi0n(RDKit) -> Any: ...
def CalcChi0v(RDKit) -> Any: ...
def CalcChi1n(RDKit) -> Any: ...
def CalcChi1v(RDKit) -> Any: ...
def CalcChi2n(RDKit) -> Any: ...
def CalcChi2v(RDKit) -> Any: ...
def CalcChi3n(RDKit) -> Any: ...
def CalcChi3v(RDKit) -> Any: ...
def CalcChi4n(RDKit) -> Any: ...
def CalcChi4v(RDKit) -> Any: ...
def CalcChiNn(*args, **kwargs) -> Any: ...
def CalcChiNv(*args, **kwargs) -> Any: ...
def CalcCoulombMat(RDKit) -> Any: ...
def CalcCrippenDescriptors(RDKit) -> Any: ...
def CalcEEMcharges(RDKit) -> Any: ...
def CalcEccentricity(RDKit) -> Any: ...
def CalcExactMolWt(RDKit) -> Any: ...
def CalcFractionCSP3(RDKit) -> Any: ...
def CalcGETAWAY(RDKit) -> Any: ...
def CalcHallKierAlpha(RDKit) -> Any: ...
def CalcInertialShapeFactor(RDKit) -> Any: ...
def CalcKappa1(RDKit) -> Any: ...
def CalcKappa2(RDKit) -> Any: ...
def CalcKappa3(RDKit) -> Any: ...
def CalcLabuteASA(RDKit) -> Any: ...
def CalcMORSE(RDKit) -> Any: ...
def CalcMolFormula(RDKit) -> Any: ...
def CalcNPR1(RDKit) -> Any: ...
def CalcNPR2(RDKit) -> Any: ...
def CalcNumAliphaticCarbocycles(RDKit) -> Any: ...
def CalcNumAliphaticHeterocycles(RDKit) -> Any: ...
def CalcNumAliphaticRings(RDKit) -> Any: ...
def CalcNumAmideBonds(RDKit) -> Any: ...
def CalcNumAromaticCarbocycles(RDKit) -> Any: ...
def CalcNumAromaticHeterocycles(RDKit) -> Any: ...
def CalcNumAromaticRings(RDKit) -> Any: ...
def CalcNumAtomStereoCenters(RDKit) -> Any: ...
def CalcNumAtoms(RDKit) -> Any: ...
def CalcNumBridgeheadAtoms(RDKit) -> Any: ...
def CalcNumHBA(RDKit) -> Any: ...
def CalcNumHBD(RDKit) -> Any: ...
def CalcNumHeavyAtoms(RDKit) -> Any: ...
def CalcNumHeteroatoms(RDKit) -> Any: ...
def CalcNumHeterocycles(RDKit) -> Any: ...
def CalcNumLipinskiHBA(RDKit) -> Any: ...
def CalcNumLipinskiHBD(RDKit) -> Any: ...
def CalcNumRings(RDKit) -> Any: ...
@overload
def CalcNumRotatableBonds(RDKit, bool) -> Any: ...
@overload
def CalcNumRotatableBonds(RDKit) -> Any: ...
def CalcNumSaturatedCarbocycles(RDKit) -> Any: ...
def CalcNumSaturatedHeterocycles(RDKit) -> Any: ...
def CalcNumSaturatedRings(RDKit) -> Any: ...
def CalcNumSpiroAtoms(RDKit) -> Any: ...
def CalcNumUnspecifiedAtomStereoCenters(RDKit) -> Any: ...
def CalcPBF(RDKit) -> Any: ...
def CalcPMI1(RDKit) -> Any: ...
def CalcPMI2(RDKit) -> Any: ...
def CalcPMI3(RDKit) -> Any: ...
def CalcPhi(RDKit) -> Any: ...
def CalcRDF(RDKit) -> Any: ...
def CalcRadiusOfGyration(RDKit) -> Any: ...
def CalcSpherocityIndex(RDKit) -> Any: ...
def CalcTPSA(RDKit) -> Any: ...
def CalcWHIM(RDKit) -> Any: ...
def CustomProp_VSA_(*args, **kwargs) -> Any: ...
def GetAtomFeatures(*args, **kwargs) -> Any: ...
def GetAtomPairAtomCode(RDKit) -> Any: ...
def GetAtomPairCode(*args, **kwargs) -> Any: ...
def GetAtomPairFingerprint(RDKit) -> Any: ...
def GetConnectivityInvariants(RDKit) -> Any: ...
def GetFeatureInvariants(RDKit) -> Any: ...
def GetHashedAtomPairFingerprint(RDKit) -> Any: ...
def GetHashedAtomPairFingerprintAsBitVect(RDKit) -> Any: ...
def GetHashedMorganFingerprint(*args, **kwargs) -> Any: ...
def GetHashedTopologicalTorsionFingerprint(RDKit) -> Any: ...
def GetHashedTopologicalTorsionFingerprintAsBitVect(RDKit) -> Any: ...
def GetMACCSKeysFingerprint(RDKit) -> Any: ...
def GetMorganFingerprint(*args, **kwargs) -> Any: ...
def GetMorganFingerprintAsBitVect(*args, **kwargs) -> Any: ...
def GetTopologicalTorsionFingerprint(RDKit) -> Any: ...
def GetUSR(RDKit) -> Any: ...
def GetUSRCAT(RDKit) -> Any: ...
def GetUSRDistributions(boost) -> Any: ...
def GetUSRDistributionsFromPoints(*args, **kwargs) -> Any: ...
def GetUSRFromDistributions(boost) -> Any: ...
def GetUSRScore(*args, **kwargs) -> Any: ...
def MQNs_(RDKit) -> Any: ...
def MakePropertyRangeQuery(*args, **kwargs) -> Any: ...
def PEOE_VSA_(RDKit) -> Any: ...
def SMR_VSA_(RDKit) -> Any: ...
def SlogP_VSA_(RDKit) -> Any: ...
def _CalcCrippenContribs(RDKit) -> Any: ...
def _CalcLabuteASAContribs(RDKit) -> Any: ...
def _CalcMolWt(RDKit) -> Any: ...
def _CalcTPSAContribs(RDKit) -> Any: ...



from rdkit.ML.Data import SplitData as SplitData
from rdkit.ML.KNN import DistFunctions as DistFunctions
from rdkit.ML.KNN.KNNClassificationModel import (
    KNNClassificationModel as KNNClassificationModel,
)
from rdkit.ML.KNN.KNNRegressionModel import KNNRegressionModel as KNNRegressionModel

def makeClassificationModel(numNeigh, attrs, distFunc): ...
def makeRegressionModel(numNeigh, attrs, distFunc): ...
def CrossValidate(knnMod, testExamples, appendExamples: int = ...): ...
def CrossValidationDriver(
    examples,
    attrs,
    nPossibleValues,
    numNeigh,
    modelBuilder=...,
    distFunc=...,
    holdOutFrac: float = ...,
    silent: int = ...,
    calcTotalError: int = ...,
    **kwargs
): ...



from _typeshed import Incomplete
from rdkit.VLib.Node import VLibNode as VLibNode

class TransformNode(VLibNode):
    _func: Incomplete
    def __init__(self, func: Incomplete | None = ..., **kwargs) -> None: ...
    def next(self): ...

def _runDoctests(verbose: Incomplete | None = ...) -> None: ...



from _typeshed import Incomplete
from rdkit.Chem.Draw.canvasbase import CanvasBase as CanvasBase

have_cairocffi: bool
have_pango: bool
ffi: Incomplete
defaultLibs: Incomplete
envVar: Incomplete
envVarSet: bool
libName: Incomplete
libPath: Incomplete
importError: bool
scriptPattern: Incomplete

class Canvas(CanvasBase):
    image: Incomplete
    imageType: Incomplete
    ctx: Incomplete
    size: Incomplete
    surface: Incomplete
    fileName: Incomplete
    def __init__(
        self,
        image: Incomplete | None = ...,
        size: Incomplete | None = ...,
        ctx: Incomplete | None = ...,
        imageType: Incomplete | None = ...,
        fileName: Incomplete | None = ...,
    ) -> None: ...
    def flush(self): ...
    def _doLine(self, p1, p2, **kwargs) -> None: ...
    def addCanvasLine(
        self, p1, p2, color=..., color2: Incomplete | None = ..., **kwargs
    ) -> None: ...
    def _addCanvasText1(self, text, pos, font, color=..., **kwargs): ...
    def _addCanvasText2(self, text, pos, font, color=..., **kwargs): ...
    def addCanvasText(self, text, pos, font, color=..., **kwargs): ...
    def addCanvasPolygon(
        self, ps, color=..., fill: bool = ..., stroke: bool = ..., **kwargs
    ) -> None: ...
    def addCanvasDashedWedge(
        self, p1, p2, p3, dash=..., color=..., color2: Incomplete | None = ..., **kwargs
    ) -> None: ...
    def addCircle(
        self,
        center,
        radius,
        color=...,
        fill: bool = ...,
        stroke: bool = ...,
        alpha: float = ...,
        **kwargs
    ) -> None: ...



from typing import Any, ClassVar

from rdkit.Chem import Mol
from rdkit.Geometry import Point3D

class EmbedParameters:
    __instance_size__: ClassVar[int] = ...
    ETversion: Any
    boundsMatForceScaling: Any
    boxSizeMult: Any
    clearConfs: Any
    embedFragmentsSeparately: Any
    enforceChirality: Any
    forceTransAmides: Any
    ignoreSmoothingFailures: Any
    maxIterations: Any
    numThreads: Any
    numZeroFail: Any
    onlyHeavyAtomsForRMS: Any
    optimizerForceTol: Any
    pruneRmsThresh: Any  # used to filter multiple conformations
    randNegEig: Any
    randomSeed: Any
    useBasicKnowledge: Any
    useExpTorsionAnglePrefs: Any
    useMacrocycleTorsions: Any
    useRandomCoords: Any
    useSmallRingTorsions: Any
    useSymmetryForPruning: Any
    verbose: Any
    @classmethod
    def __init__(cls, *args, **kwargs) -> None: ...
    @classmethod
    def SetBoundsMat(cls, RDKit, boost) -> Any: ...
    @classmethod
    def SetCPCI(cls, RDKit, boost) -> Any: ...
    @classmethod
    def __reduce__(cls) -> Any: ...

def ETDG() -> Any: ...
def ETKDG() -> Any: ...
def ETKDGv2() -> Any: ...
def ETKDGv3() -> Any: ...
def EmbedMolecule(
    mol: Mol,
    maxAttempts: int = 0,
    randomSeed: int = -1,
    clearConfs: bool = True,
    useRandomCoords: bool = False,
    boxSizeMult: float = 2.0,
    randNegEig: bool = True,
    numZeroFail: int = 1,
    coordMap: dict[int, Point3D] = {},
    forceTol: float = 0.001,
    ignoreSmoothingFailures: bool = False,
    enforceChirality: bool = True,
    useExpTorsionAnglePrefs: bool = True,
    useBasicKnowledge: bool = True,
    printExpTorsionAngles: bool = False,
    useSmallRingTorsions: bool = False,
    useMacrocycleTorsions: bool = False,
    ETversion: int = 1,
) -> int: ...
def EmbedMultipleConfs(*args, **kwargs) -> Any: ...
def GetMoleculeBoundsMatrix(RDKit) -> Any: ...
def KDG() -> Any: ...
def srETKDGv3() -> Any: ...



from _typeshed import Incomplete
from rdkit.ML.Data import cQuantize as cQuantize
from rdkit.ML.InfoTheory import entropy as entropy

hascQuantize: int
_float_tol: float

def feq(v1, v2, tol=...): ...
def FindVarQuantBound(vals, results, nPossibleRes): ...
def _GenVarTable(vals, cuts, starts, results, nPossibleRes): ...
def _PyRecurseOnBounds(
    vals, cuts, which, starts, results, nPossibleRes, varTable: Incomplete | None = ...
): ...
def _NewPyRecurseOnBounds(
    vals, cuts, which, starts, results, nPossibleRes, varTable: Incomplete | None = ...
): ...
def _NewPyFindStartPoints(sortVals, sortResults, nData): ...
def FindVarMultQuantBounds(vals, nBounds, results, nPossibleRes): ...

_RecurseOnBounds: Incomplete
_FindStartPoints: Incomplete
_RecurseOnBounds = _NewPyRecurseOnBounds
_FindStartPoints = _NewPyFindStartPoints



from typing import Any, ClassVar

from typing import overload

class AtomCompare:
    CompareAny: ClassVar[AtomCompare] = ...
    CompareAnyHeavyAtom: ClassVar[AtomCompare] = ...
    CompareElements: ClassVar[AtomCompare] = ...
    CompareIsotopes: ClassVar[AtomCompare] = ...
    names: ClassVar[dict] = ...
    values: ClassVar[dict] = ...
    __slots__: ClassVar[tuple] = ...

class BondCompare:
    CompareAny: ClassVar[BondCompare] = ...
    CompareOrder: ClassVar[BondCompare] = ...
    CompareOrderExact: ClassVar[BondCompare] = ...
    names: ClassVar[dict] = ...
    values: ClassVar[dict] = ...
    __slots__: ClassVar[tuple] = ...

class MCSAtomCompare:
    __instance_size__: ClassVar[int] = ...
    @classmethod
    def __init__(cls, *args, **kwargs) -> None: ...
    @classmethod
    def CheckAtomCharge(cls, *args, **kwargs) -> Any: ...
    @classmethod
    def CheckAtomChirality(cls, *args, **kwargs) -> Any: ...
    @classmethod
    def CheckAtomRingMatch(cls, *args, **kwargs) -> Any: ...
    @classmethod
    def compare(cls, *args, **kwargs) -> Any: ...
    @classmethod
    def __call__(cls, *args, **kwargs) -> Any: ...
    @classmethod
    def __reduce__(cls) -> Any: ...

class MCSAtomCompareParameters:
    __instance_size__: ClassVar[int] = ...
    CompleteRingsOnly: Any
    MatchChiralTag: Any
    MatchFormalCharge: Any
    MatchIsotope: Any
    MatchValences: Any
    MaxDistance: Any
    RingMatchesRingOnly: Any
    @classmethod
    def __init__(cls, *args, **kwargs) -> None: ...
    @classmethod
    def __reduce__(cls) -> Any: ...

class MCSBondCompare:
    __instance_size__: ClassVar[int] = ...
    @classmethod
    def __init__(cls, *args, **kwargs) -> None: ...
    @classmethod
    def CheckBondRingMatch(cls, *args, **kwargs) -> Any: ...
    @classmethod
    def CheckBondStereo(cls, *args, **kwargs) -> Any: ...
    @classmethod
    def compare(cls, *args, **kwargs) -> Any: ...
    @classmethod
    def __call__(cls, *args, **kwargs) -> Any: ...
    @classmethod
    def __reduce__(cls) -> Any: ...

class MCSBondCompareParameters:
    __instance_size__: ClassVar[int] = ...
    CompleteRingsOnly: Any
    MatchFusedRings: Any
    MatchFusedRingsStrict: Any
    MatchStereo: Any
    RingMatchesRingOnly: Any
    @classmethod
    def __init__(cls, *args, **kwargs) -> None: ...
    @classmethod
    def __reduce__(cls) -> Any: ...

class MCSParameters:
    __instance_size__: ClassVar[int] = ...
    AtomCompareParameters: Any
    AtomTyper: Any
    BondCompareParameters: Any
    BondTyper: Any
    InitialSeed: Any
    MaximizeBonds: Any
    ProgressCallback: Any
    Threshold: Any
    Timeout: Any
    Verbose: Any
    @classmethod
    def __init__(cls, *args, **kwargs) -> None: ...
    @classmethod
    def SetAtomTyper(cls, *args, **kwargs) -> Any: ...
    @classmethod
    def SetBondTyper(cls, *args, **kwargs) -> Any: ...
    @classmethod
    def __reduce__(cls) -> Any: ...

class MCSProgress:
    __instance_size__: ClassVar[int] = ...
    @classmethod
    def __init__(cls, *args, **kwargs) -> None: ...
    @classmethod
    def callback(cls, *args, **kwargs) -> Any: ...
    @classmethod
    def __call__(cls, *args, **kwargs) -> Any: ...
    @classmethod
    def __reduce__(cls) -> Any: ...

class MCSProgressData:
    __instance_size__: ClassVar[int] = ...
    @classmethod
    def __init__(cls, *args, **kwargs) -> None: ...
    @classmethod
    def __reduce__(cls) -> Any: ...
    @property
    def numAtoms(self) -> Any: ...
    @property
    def numBonds(self) -> Any: ...
    @property
    def seedProcessed(self) -> Any: ...

class MCSResult:
    @classmethod
    def __init__(cls, *args, **kwargs) -> None: ...
    @classmethod
    def __reduce__(cls) -> Any: ...
    @property
    def canceled(self) -> Any: ...
    @property
    def numAtoms(self) -> Any: ...
    @property
    def numBonds(self) -> Any: ...
    @property
    def queryMol(self) -> Any: ...
    @property
    def smartsString(self) -> Any: ...

class RingCompare:
    IgnoreRingFusion: ClassVar[RingCompare] = ...
    PermissiveRingFusion: ClassVar[RingCompare] = ...
    StrictRingFusion: ClassVar[RingCompare] = ...
    names: ClassVar[dict] = ...
    values: ClassVar[dict] = ...
    __slots__: ClassVar[tuple] = ...

@overload
def FindMCS(boost) -> Any: ...
@overload
def FindMCS(boost, RDKit) -> Any: ...



from _typeshed import Incomplete
from rdkit import DataStructs as DataStructs
from rdkit.Dbase import DbModule as DbModule
from rdkit.ML import CompositeRun as CompositeRun, ScreenComposite as ScreenComposite
from rdkit.ML.Composite import Composite as Composite
from rdkit.ML.Data import DataUtils as DataUtils, SplitData as SplitData
from rdkit.utils import listutils as listutils

_runDetails: Incomplete
__VERSION_STRING: str
_verbose: int

def message(msg) -> None: ...
def testall(composite, examples, badExamples=...): ...
def GetCommandLine(details): ...
def RunOnData(
    details,
    data,
    progressCallback: Incomplete | None = ...,
    saveIt: int = ...,
    setDescNames: int = ...,
): ...
def RunIt(
    details,
    progressCallback: Incomplete | None = ...,
    saveIt: int = ...,
    setDescNames: int = ...,
): ...
def ShowVersion(includeArgs: int = ...) -> None: ...
def Usage() -> None: ...
def SetDefaults(runDetails: Incomplete | None = ...): ...
def ParseArgs(runDetails) -> None: ...



from _typeshed import Incomplete
from rdkit import RDConfig as RDConfig
from rdkit.Chem.Draw.canvasbase import CanvasBase as CanvasBase

faceMap: Incomplete

def convertColor(color): ...

class Canvas(CanvasBase):
    fontScale: float
    image: Incomplete
    draw: Incomplete
    size: Incomplete
    drawType: Incomplete
    fileName: Incomplete
    def __init__(
        self,
        img: Incomplete | None = ...,
        imageType: Incomplete | None = ...,
        fileName: Incomplete | None = ...,
        size: Incomplete | None = ...,
    ) -> None: ...
    def _doLine(self, p1, p2, pen, **kwargs) -> None: ...
    def addCanvasLine(
        self, p1, p2, color=..., color2: Incomplete | None = ..., **kwargs
    ) -> None: ...
    def addCanvasText(self, text, pos, font, color=..., **kwargs): ...
    def addCanvasPolygon(
        self, ps, color=..., fill: bool = ..., stroke: bool = ..., **kwargs
    ) -> None: ...
    def addCanvasDashedWedge(
        self, p1, p2, p3, dash=..., color=..., color2: Incomplete | None = ..., **kwargs
    ) -> None: ...
    def flush(self) -> None: ...



from _typeshed import Incomplete
from rdkit import DataStructs as DataStructs
from rdkit.DataStructs.TopNContainer import TopNContainer as TopNContainer

class GenericPicker:
    _picks: Incomplete
    def MakePicks(self, force: bool = ...) -> None: ...
    def __len__(self) -> int: ...
    def __getitem__(self, which): ...

class TopNOverallPicker(GenericPicker):
    numToPick: Incomplete
    probes: Incomplete
    data: Incomplete
    simMetric: Incomplete
    _picks: Incomplete
    def __init__(
        self,
        numToPick: int = ...,
        probeFps: Incomplete | None = ...,
        dataSet: Incomplete | None = ...,
        simMetric=...,
    ) -> None: ...
    def MakePicks(self, force: bool = ...) -> None: ...

class SpreadPicker(GenericPicker):
    numToPick: Incomplete
    probes: Incomplete
    data: Incomplete
    simMetric: Incomplete
    expectPickles: Incomplete
    onlyNames: Incomplete
    _picks: Incomplete
    def __init__(
        self,
        numToPick: int = ...,
        probeFps: Incomplete | None = ...,
        dataSet: Incomplete | None = ...,
        simMetric=...,
        expectPickles: bool = ...,
        onlyNames: bool = ...,
    ) -> None: ...
    def MakePicks(self, force: bool = ..., silent: bool = ...) -> None: ...

def _runDoctests(verbose: Incomplete | None = ...) -> None: ...



from .charge import (
    ACID_BASE_PAIRS as ACID_BASE_PAIRS,
    CHARGE_CORRECTIONS as CHARGE_CORRECTIONS,
    Reionizer as Reionizer,
    Uncharger as Uncharger,
)
from .fragment import (
    FragmentRemover as FragmentRemover,
    LargestFragmentChooser as LargestFragmentChooser,
    PREFER_ORGANIC as PREFER_ORGANIC,
)
from .metal import MetalDisconnector as MetalDisconnector
from .normalize import (
    MAX_RESTARTS as MAX_RESTARTS,
    NORMALIZATIONS as NORMALIZATIONS,
    Normalizer as Normalizer,
)
from .tautomer import (
    MAX_TAUTOMERS as MAX_TAUTOMERS,
    TAUTOMER_SCORES as TAUTOMER_SCORES,
    TAUTOMER_TRANSFORMS as TAUTOMER_TRANSFORMS,
    TautomerCanonicalizer as TautomerCanonicalizer,
    TautomerEnumerator as TautomerEnumerator,
)
from .utils import memoized_property as memoized_property
from _typeshed import Incomplete
from rdkit import Chem as Chem

log: Incomplete

class Standardizer:
    normalizations: Incomplete
    acid_base_pairs: Incomplete
    charge_corrections: Incomplete
    tautomer_transforms: Incomplete
    tautomer_scores: Incomplete
    max_restarts: Incomplete
    max_tautomers: Incomplete
    prefer_organic: Incomplete
    def __init__(
        self,
        normalizations=...,
        acid_base_pairs=...,
        charge_corrections=...,
        tautomer_transforms=...,
        tautomer_scores=...,
        max_restarts=...,
        max_tautomers=...,
        prefer_organic=...,
    ) -> None: ...
    def __call__(self, mol): ...
    def standardize(self, mol): ...
    def tautomer_parent(self, mol, skip_standardize: bool = ...): ...
    def fragment_parent(self, mol, skip_standardize: bool = ...): ...
    def stereo_parent(self, mol, skip_standardize: bool = ...): ...
    def isotope_parent(self, mol, skip_standardize: bool = ...): ...
    def charge_parent(self, mol, skip_standardize: bool = ...): ...
    def super_parent(self, mol, skip_standardize: bool = ...): ...
    def standardize_with_parents(self, mol): ...
    def disconnect_metals(self): ...
    def normalize(self): ...
    def reionize(self): ...
    def uncharge(self): ...
    def remove_fragments(self): ...
    def largest_fragment(self): ...
    def enumerate_tautomers(self): ...
    def canonicalize_tautomer(self): ...

def standardize_smiles(smiles): ...
def enumerate_tautomers_smiles(smiles): ...
def canonicalize_tautomer_smiles(smiles): ...



from _typeshed import Incomplete
from rdkit import RDConfig as RDConfig

TEST_FAILED: int
TEST_PASSED: int
BUILD_TYPE_ENVVAR: str

def isDebugBuild(): ...
def RunTest(exeName, args, extras): ...
def RunScript(script, doLongTests, verbose): ...
def ReportResults(script, failedTests, nTests, runTime, verbose, dest) -> None: ...

class _RedirectStream:
    _stream: Incomplete
    _new_target: Incomplete
    _old_targets: Incomplete
    def __init__(self, new_target) -> None: ...
    def __enter__(self): ...
    def __exit__(self, exctype, excinst, exctb) -> None: ...

class redirect_stdout(_RedirectStream):
    _stream: str

class redirect_stderr(_RedirectStream):
    _stream: str

class OutputRedirectC:
    outfiles: Incomplete
    combine: Incomplete
    mode: Incomplete
    saved_streams: Incomplete
    fds: Incomplete
    saved_fds: Incomplete
    null_fds: Incomplete
    null_streams: Incomplete
    def __init__(self, stdout=..., stderr=..., mode: str = ...) -> None: ...
    def __enter__(self) -> None: ...
    def __exit__(self, *args): ...



def CalcROC(scores, col): ...
def CalcAUC(scores, col): ...
def _RIEHelper(scores, col, alpha): ...
def CalcRIE(scores, col, alpha): ...
def CalcBEDROC(scores, col, alpha): ...
def CalcEnrichment(scores, col, fractions): ...



from _typeshed import Incomplete

class MCSResult:
    numAtoms: Incomplete
    numBonds: Incomplete
    smarts: Incomplete
    completed: Incomplete
    def __init__(self, obj) -> None: ...
    def __nonzero__(self): ...
    def __repr__(self): ...
    def __str__(self): ...

def FindMCS(
    mols,
    minNumAtoms: int = ...,
    maximize=...,
    atomCompare=...,
    bondCompare=...,
    matchValences=...,
    ringMatchesRingOnly: bool = ...,
    completeRingsOnly: bool = ...,
    timeout=...,
    threshold: Incomplete | None = ...,
): ...
